@@ -18,6 +18,7 @@
 # along
 # with this program; if not, see <http://www.gnu.org/licenses>.
 
+from collections import OrderedDict
 import logging
 import copy
 from functools import reduce
@@ -27,7 +28,7 @@ import types
 import traceback
 import shlex
 import subprocess  # nosec - internal
-from collections import OrderedDict
+import warnings
 
 from lava_common.decorators import nottest
 from lava_common.timeout import Timeout
@@ -38,11 +39,11 @@ from lava_common.exceptions import (
     TestError,
     LAVATimeoutError,
 )
-from lava_dispatcher.log import YAMLLogger
+from lava_common.log import YAMLLogger
 from lava_dispatcher.utils.strings import seconds_to_str
 
 
-class InternalObject:  # pylint: disable=too-few-public-methods
+class InternalObject:
     """
     An object within the dispatcher pipeline which should not be included in
     the description of the pipeline.
@@ -51,7 +52,7 @@ class InternalObject:  # pylint: disable=too-few-public-methods
     pass
 
 
-class Pipeline:  # pylint: disable=too-many-instance-attributes
+class Pipeline:
     """
     Pipelines ensure that actions are run in the correct sequence whilst
     allowing for retries and other requirements.
@@ -75,7 +76,7 @@ class Pipeline:  # pylint: disable=too-many-instance-attributes
                 )
             self.parent = parent
 
-    def _check_action(self, action):  # pylint: disable=no-self-use
+    def _check_action(self, action):
         if not action:
             raise LAVABug("Need an action to add to the pipeline, not None.")
         elif not issubclass(type(action), Action):
@@ -83,7 +84,7 @@ class Pipeline:  # pylint: disable=too-many-instance-attributes
         # if isinstance(action, DiagnosticAction):
         #     raise LAVABug("Diagnostic actions need to be triggered, not added to a pipeline.")
 
-    def add_action(self, action, parameters=None):  # pylint: disable=too-many-branches
+    def add_action(self, action, parameters=None):
         self._check_action(action)
         self.actions.append(action)
         # FIXME: if this is only happening in unit test, this has to be fixed later on
@@ -113,8 +114,6 @@ class Pipeline:  # pylint: disable=too-many-instance-attributes
             # Then job level overrides
             timeouts.append(self.job.parameters.get("timeouts", {}))
 
-        # pylint: disable=invalid-name
-
         def dict_merge_get(dicts, key):
             value = None
             for d in dicts:
@@ -131,7 +130,7 @@ class Pipeline:  # pylint: disable=too-many-instance-attributes
         # 1/ the global action timeout
         # 2/ the individual action timeout
         # 3/ the action block timeout
-        # pylint: disable=protected-access
+
         action._override_action_timeout(dict_merge_get(timeouts, "action"))
         action._override_action_timeout(
             subdict_merge_get(timeouts, "actions", action.name)
@@ -159,7 +158,9 @@ class Pipeline:  # pylint: disable=too-many-instance-attributes
                 cls = str(type(action))[8:-2].replace("lava_dispatcher.", "")
                 current = {"class": cls, "name": action.name}
             if action.pipeline is not None:
-                current["pipeline"] = action.pipeline.describe(verbose)
+                sub_desc = action.pipeline.describe(verbose)
+                if sub_desc:
+                    current["pipeline"] = sub_desc
             desc.append(current)
         return desc
 
@@ -320,7 +321,7 @@ class CommandLogger:
             self.write("\n")
 
 
-class Action:  # pylint: disable=too-many-instance-attributes,too-many-public-methods
+class Action:
     def __init__(self):
         """
         Actions get added to pipelines by calling the
@@ -339,12 +340,11 @@ class Action:  # pylint: disable=too-many-instance-attributes,too-many-public-me
         # subsequently except by RetryCommand.
         self.level = None
         self.pipeline = None
-        self.internal_pipeline = None
         self.__parameters__ = {}
         self.__errors__ = []
         self.job = None
         self.logger = logging.getLogger("dispatcher")
-        self.__results__ = OrderedDict()
+        self.__results__ = {}
         self.timeout = Timeout(self.name, exception=self.timeout_exception)
         # unless the strategy or the job parameters change this, do not retry
         self.max_retries = 1
@@ -389,15 +389,15 @@ class Action:  # pylint: disable=too-many-instance-attributes,too-many-public-me
 
     @classmethod
     def select(cls, name):
-        for subclass in cls.__subclasses__():  # pylint: disable=no-member
+        for subclass in cls.__subclasses__():
             if subclass.name == name:
                 return subclass
         raise JobError('Cannot find action named "%s"' % name)
 
     @property
     def errors(self):
-        if self.internal_pipeline:
-            return self.__errors__ + self.internal_pipeline.errors
+        if self.pipeline:
+            return self.__errors__ + self.pipeline.errors
         else:
             return self.__errors__
 
@@ -496,13 +496,13 @@ class Action:  # pylint: disable=too-many-instance-attributes,too-many-public-me
         if not self.name:
             self.errors = "%s action has no name set" % self
         # have already checked that self.name is not None, but pylint gets confused.
-        if " " in self.name:  # pylint: disable=unsupported-membership-test
+        if " " in self.name:
             self.errors = (
                 "Whitespace must not be used in action names, only descriptions or summaries: %s"
                 % self.name
             )
 
-        if "_" in self.name:  # pylint: disable=unsupported-membership-test
+        if "_" in self.name:
             self.errors = "Use - instead of _ in action names: %s" % self.name
 
         if not self.summary:
@@ -515,8 +515,8 @@ class Action:  # pylint: disable=too-many-instance-attributes,too-many-public-me
             self.errors = "action %s (%s) has no section set" % (self.name, self)
 
         # Collect errors from internal pipeline actions
-        if self.internal_pipeline:
-            self.internal_pipeline.validate_actions()
+        if self.pipeline:
+            self.pipeline.validate_actions()
 
     def populate(self, parameters):
         """
@@ -549,8 +549,7 @@ class Action:  # pylint: disable=too-many-instance-attributes,too-many-public-me
         if not isinstance(command_list, list):
             raise LAVABug("commands to parsed_command need to be a list")
         log = ""
-        # nice is assumed to always exist (coreutils)
-        command_list = ["nice"] + [str(s) for s in command_list]
+        command_list = [str(s) for s in command_list]
         self.logger.debug("%s", " ".join(command_list))
         try:
             log = subprocess.check_output(  # nosec - internal
@@ -612,6 +611,7 @@ class Action:  # pylint: disable=too-many-instance-attributes,too-many-public-me
         :param: allow_fail - if True, do not raise a JobError when the command fail (return non 0)
         :param: error_msg - the exception message.
         :param: cwd - the current working directory for this command
+        :return: return code of the command
         """
         # Build the command list (adding 'nice' at the front)
         if isinstance(command_list, str):
@@ -662,6 +662,7 @@ class Action:  # pylint: disable=too-many-instance-attributes,too-many-public-me
         if ret != 0 and not allow_fail:
             self.logger.error("Unable to run 'nice' '%s'", command_list)
             raise self.command_exception(error_msg)
+        return ret
 
     def run_command(self, command_list, allow_silent=False, allow_fail=False, cwd=None):
         """
@@ -686,12 +687,13 @@ class Action:  # pylint: disable=too-many-instance-attributes,too-many-public-me
         On failure (command exited non-zero), sets self.errors.
         If allow_silent is True, returns False, else returns the command output.
         """
+        warnings.warn("run_command should be replaced by run_cmd", DeprecationWarning)
+
         # FIXME: add option to only check stdout or stderr for failure output
         if not isinstance(command_list, list):
             raise LAVABug("commands to run_command need to be a list")
         log = None
-        # nice is assumed to always exist (coreutils)
-        command_list = ["nice"] + [str(s) for s in command_list]
+        command_list = [str(s) for s in command_list]
         self.logger.debug("%s", " ".join(command_list))
         try:
             log = subprocess.check_output(  # nosec - managed
@@ -788,8 +790,8 @@ class Action:  # pylint: disable=too-many-instance-attributes,too-many-public-me
         all exceptions possible from the command and re-raise
         """
         self.call_protocols()
-        if self.internal_pipeline:
-            return self.internal_pipeline.run_actions(connection, max_end_time)
+        if self.pipeline:
+            return self.pipeline.run_actions(connection, max_end_time)
         if connection:
             connection.timeout = self.connection_timeout
         return connection
@@ -807,8 +809,8 @@ class Action:  # pylint: disable=too-many-instance-attributes,too-many-public-me
         Use contextmanagers or signal handlers to clean up any resources when there are no errors,
         instead of using cleanup().
         """
-        if self.internal_pipeline:
-            self.internal_pipeline.cleanup(connection)
+        if self.pipeline:
+            self.pipeline.cleanup(connection)
 
     def explode(self):
         """
@@ -831,7 +833,7 @@ class Action:  # pylint: disable=too-many-instance-attributes,too-many-public-me
         # noinspection PySetFunctionToLiteral
         skip_set = set(
             [
-                "internal_pipeline",
+                "pipeline",
                 "job",
                 "logger",
                 "pipeline",
@@ -854,7 +856,7 @@ class Action:  # pylint: disable=too-many-instance-attributes,too-many-public-me
                     "name": self.timeout.name,
                 }
             elif attr == "url":
-                data["url"] = self.url.geturl()  # pylint: disable=no-member
+                data["url"] = self.url.geturl()
             elif attr == "vcs":
                 data[attr] = getattr(self, attr).url
             elif attr == "protocols":
@@ -909,9 +911,7 @@ class Action:  # pylint: disable=too-many-instance-attributes,too-many-public-me
         """
         params = parameters if parameters else self.parameters
         namespace = params["namespace"]
-        value = (
-            self.data.get(namespace, {}).get(action, {}).get(label, {}).get(key)
-        )  # pylint: disable=no-member
+        value = self.data.get(namespace, {}).get(action, {}).get(label, {}).get(key)
         if value is None:
             return None
         return copy.deepcopy(value) if deepcopy else value
@@ -934,7 +934,7 @@ class Action:  # pylint: disable=too-many-instance-attributes,too-many-public-me
         namespace = params["namespace"]
         if not label or not key:
             raise LAVABug("Invalid call to set_namespace_data: %s" % action)
-        self.data.setdefault(namespace, {})  # pylint: disable=no-member
+        self.data.setdefault(namespace, {})
         self.data[namespace].setdefault(action, {})
         self.data[namespace][action].setdefault(label, {})
         self.data[namespace][action][label][key] = value
@@ -996,7 +996,7 @@ class Action:  # pylint: disable=too-many-instance-attributes,too-many-public-me
                 # allows retries without setting errors, which make the job incomplete.
                 res = "fail"
             self.logger.results(
-                {  # pylint: disable=no-member
+                {
                     "definition": "lava",
                     "namespace": self.parameters.get("namespace", "common"),
                     "case": self.name,
@@ -1015,28 +1015,47 @@ class Action:  # pylint: disable=too-many-instance-attributes,too-many-public-me
             )
 
     @nottest
-    def test_needs_deployment(self, parameters):  # pylint: disable=no-self-use
-        if parameters["namespace"] in self.job.test_info:
-            testclasses = self.job.test_info[parameters["namespace"]]
-            for testclass in testclasses:
-                if testclass["class"].needs_deployment_data():
-                    return True
+    def test_needs_deployment(self, parameters):
+        ns = parameters["namespace"]
+        for info in self.job.test_info.get(ns, []):
+            if info["class"].needs_deployment_data(info["parameters"]):
+                return True
         return False
 
     @nottest
-    def test_has_shell(self, parameters):  # pylint: disable=no-self-use
-        if parameters["namespace"] in self.job.test_info:
-            testclasses = self.job.test_info[parameters["namespace"]]
-            for testclass in testclasses:
-                if testclass["class"].has_shell():
-                    return True
+    def test_has_shell(self, parameters):
+        ns = parameters["namespace"]
+        for info in self.job.test_info.get(ns, []):
+            if info["class"].has_shell(info["parameters"]):
+                return True
         return False
 
     @nottest
-    def test_needs_overlay(self, parameters):  # pylint: disable=no-self-use
-        if parameters["namespace"] in self.job.test_info:
-            testclasses = self.job.test_info[parameters["namespace"]]
-            for testclass in testclasses:
-                if testclass["class"].needs_overlay():
-                    return True
+    def test_needs_overlay(self, parameters):
+        ns = parameters["namespace"]
+        for info in self.job.test_info.get(ns, []):
+            if info["class"].needs_overlay(info["parameters"]):
+                return True
         return False
+
+    def get_logging_info(self):
+        job_id = self.job.job_id
+        logging_url = master_cert = slave_cert = socks_proxy = ipv6 = None
+
+        handler = getattr(self.logger, "handler", None)
+        if handler:
+            logging_url = handler.logging_url
+            master_cert = handler.master_cert
+            slave_cert = handler.slave_cert
+            socks_proxy = handler.socks_proxy
+            ipv6 = handler.ipv6
+            job_id = handler.job_id
+
+        return {
+            "logging_url": logging_url,
+            "master_cert": master_cert,
+            "slave_cert": slave_cert,
+            "socks_proxy": socks_proxy,
+            "ipv6": ipv6,
+            "job_id": job_id,
+        }
