@@ -48,32 +48,14 @@ from lava_dispatcher.connections.ssh import SShSession
 from lava_dispatcher.actions.boot.environment import ExportDeviceEnvironment
 from lava_dispatcher.shell import ExpectShellSession
 
-# pylint: disable=too-many-locals,too-many-instance-attributes,superfluous-parens
-# pylint: disable=too-many-branches,too-many-statements
 
+class BootHasMixin:
+    """ Add the two methods to boot classes using it """
 
-class BootAction(RetryAction):
-    """
-    Base class for all actions which control power-on
-    and boot behaviour of a device under test.
-    The subclass selected to do the work will be the
-    subclass returning True in the accepts(device_type, image)
-    function.
-    Each new subclass needs a unit test to ensure it is
-    reliably selected for the correct job and not
-    selected for an invalid job or a job
-    accepted by a different subclass.
-
-    Boot and Test are closely related - a fail error in Boot
-    will cause subsequent Test actions to be skipped.
-    """
-
-    name = "boot"
-
-    def has_prompts(self, parameters):  # pylint: disable=no-self-use
+    def has_prompts(self, parameters):
         return "prompts" in parameters
 
-    def has_boot_finished(self, parameters):  # pylint: disable=no-self-use
+    def has_boot_finished(self, parameters):
         return "boot_finished" in parameters
 
 
@@ -104,7 +86,7 @@ class AutoLoginAction(Action):
         self.params = None
         self.booting = booting  # if a boot is expected, False for second UART or ssh.
 
-    def validate(self):  # pylint: disable=too-many-branches
+    def validate(self):
         super().validate()
         # Skip auto login if the configuration is not found
         self.method = self.parameters["method"]
@@ -339,8 +321,7 @@ class BootloaderCommandOverlay(Action):
     particular pipeline.
     addresses are read from the device configuration parameters
     bootloader_type is determined from the boot action method strategy
-    bootz or bootm is determined by boot action method type. (i.e. it is up to
-    the test writer to select the correct download file for the correct boot command.)
+    bootz or bootm is determined by deploy action kernel type.
     server_ip is calculated at runtime
     filenames are determined from the download Action.
     """
@@ -349,9 +330,9 @@ class BootloaderCommandOverlay(Action):
     description = "substitute job data into bootloader command list"
     summary = "replace placeholders with job data"
 
-    def __init__(self, method=None):
+    def __init__(self, method=None, commands=None):
         super().__init__()
-        self.commands = None
+        self.commands = commands
         self.method = method
         self.use_bootscript = False
         self.lava_mac = None
@@ -363,6 +344,8 @@ class BootloaderCommandOverlay(Action):
         if self.method is None:
             self.method = self.parameters["method"]
         device_methods = self.job.device["actions"]["boot"]["methods"]
+        if self.commands is not None:
+            return
         if self.parameters["method"] == "bootloader":
             self.commands = self.parameters["commands"]
         elif isinstance(self.parameters["commands"], list):
@@ -390,13 +373,11 @@ class BootloaderCommandOverlay(Action):
         # download-action will set ['dtb'] as tftp_path, tmpdir & filename later, in the run step.
         if "use_bootscript" in self.parameters:
             self.use_bootscript = self.parameters["use_bootscript"]
-        if "lava_mac" in self.parameters:
-            if re.match(
-                "([0-9A-F]{2}[:-]){5}([0-9A-F]{2})",
-                self.parameters["lava_mac"],
-                re.IGNORECASE,
-            ):
-                self.lava_mac = self.parameters["lava_mac"]
+
+        if "lava_mac" in device_methods[self.method]["parameters"]:
+            lava_mac = device_methods[self.method]["parameters"]["lava_mac"]
+            if re.match("([0-9A-F]{2}[:-]){5}([0-9A-F]{2})", lava_mac, re.IGNORECASE):
+                self.lava_mac = lava_mac
             else:
                 self.errors = "lava_mac is not a valid mac address"
 
@@ -441,13 +422,8 @@ class BootloaderCommandOverlay(Action):
         self.bootcommand = self.get_namespace_data(
             action="uboot-prepare-kernel", label="bootcommand", key="bootcommand"
         )
-        if not self.bootcommand:
-            if "type" in self.parameters:
-                self.logger.warning(
-                    "Using type from the boot action as the boot-command. "
-                    "Declaring a kernel type in the deploy is preferred."
-                )
-                self.bootcommand = self.parameters["type"]
+        if not self.bootcommand and "type" in self.parameters:
+            raise JobError("Kernel image type can't be determined")
         prepared_kernel = self.get_namespace_data(
             action="prepare-kernel", label="file", key="kernel"
         )
@@ -457,7 +433,6 @@ class BootloaderCommandOverlay(Action):
             )
             substitutions["{KERNEL}"] = prepared_kernel
         if self.bootcommand:
-            self.logger.debug("%s", self.job.device["parameters"])
             kernel_addr = self.job.device["parameters"][self.bootcommand]["kernel"]
             dtb_addr = self.job.device["parameters"][self.bootcommand]["dtb"]
             ramdisk_addr = self.job.device["parameters"][self.bootcommand]["ramdisk"]
@@ -579,11 +554,16 @@ class BootloaderCommandOverlay(Action):
             )
             self.logger.info("Parsed boot commands: %s", "; ".join(bootscript_commands))
             return connection
-        subs = substitute(self.commands, substitutions)
+        subs = substitute(self.commands, substitutions, drop=True)
         self.set_namespace_data(
             action="bootloader-overlay", label=self.method, key="commands", value=subs
         )
-        self.logger.info("Parsed boot commands: %s", "; ".join(subs))
+        self.logger.debug("substitutions:")
+        for k in sorted(substitutions.keys()):
+            self.logger.debug("- %s: %s", k, substitutions[k])
+        self.logger.info("Parsed boot commands:")
+        for sub in subs:
+            self.logger.info("- %s", sub)
         return connection
 
 
@@ -695,7 +675,7 @@ class OverlayUnpack(Action):
         connection.wait()
 
         cmd = self.parameters["transfer_overlay"]["download_command"]
-        ip_addr = dispatcher_ip(self.job.parameters["dispatcher"])
+        ip_addr = dispatcher_ip(self.job.parameters["dispatcher"], "http")
         connection.sendline("%s http://%s/tmp/%s" % (cmd, ip_addr, overlay_path))
         connection.wait()
 
@@ -739,7 +719,7 @@ class BootloaderInterruptAction(Action):
         self.params = self.job.device["actions"]["boot"]["methods"][self.method][
             "parameters"
         ]
-        if self.job.device.connect_command is "":
+        if self.job.device.connect_command == "":
             self.errors = "Unable to connect to device %s"
         device_methods = self.job.device["actions"]["boot"]["methods"]
         if (
@@ -782,7 +762,7 @@ class BootloaderInterruptAction(Action):
             raise LAVABug("%s started without a connection already in use" % self.name)
         connection = super().run(connection, max_end_time)
         if self.needs_interrupt:
-            connection.prompt_str = self.interrupt_prompt
+            connection.prompt_str = [self.interrupt_prompt]
             self.wait(connection)
             if self.interrupt_control_chars:
                 for char in self.interrupt_control_chars:
@@ -796,7 +776,7 @@ class BootloaderInterruptAction(Action):
             self.logger.info(
                 "Not interrupting bootloader, waiting for bootloader prompt"
             )
-            connection.prompt_str = self.bootloader_prompt
+            connection.prompt_str = [self.bootloader_prompt]
             self.wait(connection)
             self.set_namespace_data(
                 action="interrupt",
@@ -820,7 +800,9 @@ class BootloaderCommandsAction(Action):
     def __init__(self, expect_final=True, method=None):
         super().__init__()
         self.params = None
-        self.timeout = Timeout(self.name, BOOTLOADER_DEFAULT_CMD_TIMEOUT)
+        self.timeout = Timeout(
+            self.name, BOOTLOADER_DEFAULT_CMD_TIMEOUT, exception=self.timeout_exception
+        )
         self.method = method
         self.expect_final = expect_final
 
@@ -873,7 +855,7 @@ class BootloaderCommandsAction(Action):
                 raise InfrastructureError(msg)
 
         if final_message and self.expect_final:
-            connection.prompt_str = final_message
+            connection.prompt_str = [final_message]
             self.wait(connection, max_end_time)
 
         self.set_namespace_data(
