@@ -22,7 +22,6 @@ import shutil
 
 from lava_common.exceptions import InfrastructureError
 from lava_dispatcher.action import Pipeline, Action
-from lava_dispatcher.actions.boot import BootAction
 from lava_dispatcher.connections.serial import ConnectDevice
 from lava_dispatcher.logical import Boot, RetryAction
 from lava_dispatcher.power import ResetDevice
@@ -34,12 +33,9 @@ class CMSIS(Boot):
 
     compatibility = 4  # FIXME: change this to 5 and update test cases
 
-    def __init__(self, parent, parameters):
-        super().__init__(parent)
-        self.action = BootCMSIS()
-        self.action.section = self.action_type
-        self.action.job = self.job
-        parent.add_action(self.action, parameters)
+    @classmethod
+    def action(cls):
+        return BootCMSIS()
 
     @classmethod
     def accepts(cls, device, parameters):
@@ -67,17 +63,15 @@ class CMSIS(Boot):
         return True, "accepted"
 
 
-class BootCMSIS(BootAction):
+class BootCMSIS(RetryAction):
 
     name = "boot-cmsis"
     description = "boot cmsis usb image"
     summary = "boot cmsis usb image"
 
     def populate(self, parameters):
-        self.internal_pipeline = Pipeline(
-            parent=self, job=self.job, parameters=parameters
-        )
-        self.internal_pipeline.add_action(BootCMSISRetry())
+        self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
+        self.pipeline.add_action(BootCMSISRetry())
 
 
 class BootCMSISRetry(RetryAction):
@@ -96,21 +90,19 @@ class BootCMSISRetry(RetryAction):
             self.errors = "usb_mass_device unset"
 
     def populate(self, parameters):
-        self.internal_pipeline = Pipeline(
-            parent=self, job=self.job, parameters=parameters
-        )
+        self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
         method_params = self.job.device["actions"]["boot"]["methods"]["cmsis-dap"][
             "parameters"
         ]
         usb_mass_device = method_params.get("usb_mass_device")
         resets_after_flash = method_params.get("resets_after_flash", True)
         if self.job.device.hard_reset_command:
-            self.internal_pipeline.add_action(ResetDevice())
-            self.internal_pipeline.add_action(WaitDevicePathAction(usb_mass_device))
-        self.internal_pipeline.add_action(FlashCMSISAction())
+            self.pipeline.add_action(ResetDevice())
+            self.pipeline.add_action(WaitDevicePathAction(usb_mass_device))
+        self.pipeline.add_action(FlashCMSISAction())
         if resets_after_flash:
-            self.internal_pipeline.add_action(WaitUSBSerialDeviceAction())
-        self.internal_pipeline.add_action(ConnectDevice())
+            self.pipeline.add_action(WaitUSBSerialDeviceAction())
+        self.pipeline.add_action(ConnectDevice())
 
 
 class FlashCMSISAction(Action):
