@@ -21,7 +21,7 @@
 from lava_common.exceptions import ConfigurationError, InfrastructureError
 
 from lava_dispatcher.action import Action, Pipeline
-from lava_dispatcher.actions.boot import BootAction, BootloaderInterruptAction
+from lava_dispatcher.actions.boot import BootloaderInterruptAction
 from lava_dispatcher.connections.serial import ConnectDevice
 from lava_dispatcher.logical import Boot, RetryAction
 from lava_dispatcher.power import ResetDevice
@@ -34,12 +34,9 @@ class DFU(Boot):
 
     compatibility = 4  # FIXME: change this to 5 and update test cases
 
-    def __init__(self, parent, parameters):
-        super().__init__(parent)
-        self.action = BootDFU()
-        self.action.section = self.action_type
-        self.action.job = self.job
-        parent.add_action(self.action, parameters)
+    @classmethod
+    def action(cls):
+        return BootDFU()
 
     @classmethod
     def accepts(cls, device, parameters):
@@ -54,17 +51,15 @@ class DFU(Boot):
         return True, "accepted"
 
 
-class BootDFU(BootAction):
+class BootDFU(RetryAction):
 
     name = "boot-dfu-image"
     description = "boot dfu image with retry"
     summary = "boot dfu image with retry"
 
     def populate(self, parameters):
-        self.internal_pipeline = Pipeline(
-            parent=self, job=self.job, parameters=parameters
-        )
-        self.internal_pipeline.add_action(BootDFURetry())
+        self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
+        self.pipeline.add_action(BootDFURetry())
 
 
 class BootDFURetry(RetryAction):
@@ -77,18 +72,14 @@ class BootDFURetry(RetryAction):
         dfu = self.job.device["actions"]["boot"]["methods"]["dfu"]
         parameters = dfu["parameters"]
 
-        self.internal_pipeline = Pipeline(
-            parent=self, job=self.job, parameters=parameters
-        )
-        self.internal_pipeline.add_action(ConnectDevice())
-        self.internal_pipeline.add_action(ResetDevice())
+        self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
+        self.pipeline.add_action(ConnectDevice())
+        self.pipeline.add_action(ResetDevice())
         if dfu.get("implementation") == "u-boot":
-            self.internal_pipeline.add_action(
-                BootloaderInterruptAction(method="u-boot")
-            )
-            self.internal_pipeline.add_action(EnterDFU())
-        self.internal_pipeline.add_action(WaitDFUDeviceAction())
-        self.internal_pipeline.add_action(FlashDFUAction())
+            self.pipeline.add_action(BootloaderInterruptAction(method="u-boot"))
+            self.pipeline.add_action(EnterDFU())
+        self.pipeline.add_action(WaitDFUDeviceAction())
+        self.pipeline.add_action(FlashDFUAction())
 
 
 class EnterDFU(Action):
