@@ -20,9 +20,10 @@
 
 import os
 
+from lava_common.exceptions import JobError
 from lava_dispatcher.action import Pipeline, Action
 from lava_dispatcher.logical import Boot, RetryAction
-from lava_dispatcher.actions.boot import BootAction
+from lava_dispatcher.actions.boot import BootHasMixin
 from lava_dispatcher.actions.boot.environment import ExportDeviceEnvironment
 from lava_dispatcher.shell import ExpectShellSession, ShellCommand, ShellSession
 
@@ -30,12 +31,9 @@ from lava_dispatcher.shell import ExpectShellSession, ShellCommand, ShellSession
 class BootDocker(Boot):
     compatibility = 4
 
-    def __init__(self, parent, parameters):
-        super().__init__(parent)
-        self.action = BootDockerAction()
-        self.action.section = self.action_type
-        self.action.job = self.job
-        parent.add_action(self.action, parameters)
+    @classmethod
+    def action(cls):
+        return BootDockerAction()
 
     @classmethod
     def accepts(cls, device, parameters):
@@ -48,21 +46,19 @@ class BootDocker(Boot):
         return True, "accepted"
 
 
-class BootDockerAction(BootAction):
+class BootDockerAction(BootHasMixin, RetryAction):
 
     name = "boot-docker"
     description = "boot docker image"
     summary = "boot docker image"
 
     def populate(self, parameters):
-        self.internal_pipeline = Pipeline(
-            parent=self, job=self.job, parameters=parameters
-        )
-        self.internal_pipeline.add_action(BootDockerRetry())
+        self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
+        self.pipeline.add_action(BootDockerRetry())
         if self.has_prompts(parameters):
             if self.test_has_shell(parameters):
-                self.internal_pipeline.add_action(ExpectShellSession())
-                self.internal_pipeline.add_action(ExportDeviceEnvironment())
+                self.pipeline.add_action(ExpectShellSession())
+                self.pipeline.add_action(ExportDeviceEnvironment())
 
 
 class BootDockerRetry(RetryAction):
@@ -72,10 +68,8 @@ class BootDockerRetry(RetryAction):
     summary = "boot docker image"
 
     def populate(self, parameters):
-        self.internal_pipeline = Pipeline(
-            parent=self, job=self.job, parameters=parameters
-        )
-        self.internal_pipeline.add_action(CallDockerAction())
+        self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
+        self.pipeline.add_action(CallDockerAction())
 
 
 class CallDockerAction(Action):
@@ -96,16 +90,28 @@ class CallDockerAction(Action):
 
         options = self.job.device["actions"]["boot"]["methods"]["docker"]["options"]
 
+        docker_image = self.get_namespace_data(
+            action="deploy-docker", label="image", key="name"
+        )
+        if docker_image is None:
+            raise JobError("Missing deploy action before boot")
+
         if options["cpus"]:
             self.extra_options += " --cpus %s" % options["cpus"]
         if options["memory"]:
             self.extra_options += " --memory %s" % options["memory"]
         if options["privileged"]:
             self.extra_options += " --privileged"
+        for cap in options["capabilities"]:
+            self.extra_options += " --cap-add %s" % cap
         for device in options["devices"]:
             self.extra_options += " --device %s" % device
+        for network in options["networks"]:
+            self.extra_options += " --network %s" % network
         for volume in options["volumes"]:
             self.extra_options += " --volume %s" % volume
+        for extra_argument in options["extra_arguments"]:
+            self.extra_options += " " + extra_argument
 
     def run(self, connection, max_end_time):
         location = self.get_namespace_data(
