@@ -22,12 +22,12 @@
 # imported by the parser to populate the list of subclasses.
 
 from lava_dispatcher.action import Pipeline
-from lava_dispatcher.logical import Boot
+from lava_dispatcher.logical import Boot, RetryAction
 from lava_dispatcher.actions.boot import (
-    BootAction,
     AutoLoginAction,
     BootloaderCommandOverlay,
     BootloaderCommandsAction,
+    BootHasMixin,
     OverlayUnpack,
     BootloaderInterruptAction,
 )
@@ -49,12 +49,9 @@ class IPXE(Boot):
 
     compatibility = 1
 
-    def __init__(self, parent, parameters):
-        super().__init__(parent)
-        self.action = BootloaderAction()
-        self.action.section = self.action_type
-        self.action.job = self.job
-        parent.add_action(self.action, parameters)
+    @classmethod
+    def action(cls):
+        return BootloaderAction()
 
     @classmethod
     def accepts(cls, device, parameters):
@@ -66,7 +63,7 @@ class IPXE(Boot):
             return False, '"ipxe" was not in the device configuration boot methods'
 
 
-class BootloaderAction(BootAction):
+class BootloaderAction(RetryAction):
     """
     Wraps the Retry Action to allow for actions which precede
     the reset, e.g. Connect.
@@ -77,16 +74,14 @@ class BootloaderAction(BootAction):
     summary = "pass boot commands"
 
     def populate(self, parameters):
-        self.internal_pipeline = Pipeline(
-            parent=self, job=self.job, parameters=parameters
-        )
+        self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
         # customize the device configuration for this job
-        self.internal_pipeline.add_action(BootloaderCommandOverlay())
-        self.internal_pipeline.add_action(ConnectDevice())
-        self.internal_pipeline.add_action(BootloaderRetry())
+        self.pipeline.add_action(BootloaderCommandOverlay())
+        self.pipeline.add_action(ConnectDevice())
+        self.pipeline.add_action(BootloaderRetry())
 
 
-class BootloaderRetry(BootAction):
+class BootloaderRetry(BootHasMixin, RetryAction):
 
     name = "bootloader-retry"
     description = "interactive uboot retry action"
@@ -98,21 +93,19 @@ class BootloaderRetry(BootAction):
         self.force_prompt = False
 
     def populate(self, parameters):
-        self.internal_pipeline = Pipeline(
-            parent=self, job=self.job, parameters=parameters
-        )
+        self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
         # establish a new connection before trying the reset
-        self.internal_pipeline.add_action(ResetDevice())
-        self.internal_pipeline.add_action(BootloaderInterruptAction())
+        self.pipeline.add_action(ResetDevice())
+        self.pipeline.add_action(BootloaderInterruptAction())
         # need to look for Hit any key to stop autoboot
-        self.internal_pipeline.add_action(BootloaderCommandsAction())
+        self.pipeline.add_action(BootloaderCommandsAction())
         if self.has_prompts(parameters):
-            self.internal_pipeline.add_action(AutoLoginAction())
+            self.pipeline.add_action(AutoLoginAction())
             if self.test_has_shell(parameters):
-                self.internal_pipeline.add_action(ExpectShellSession())
+                self.pipeline.add_action(ExpectShellSession())
                 if "transfer_overlay" in parameters:
-                    self.internal_pipeline.add_action(OverlayUnpack())
-                self.internal_pipeline.add_action(ExportDeviceEnvironment())
+                    self.pipeline.add_action(OverlayUnpack())
+                self.pipeline.add_action(ExportDeviceEnvironment())
 
     def validate(self):
         super().validate()
