@@ -25,19 +25,15 @@ import subprocess
 
 from lava_dispatcher.action import Pipeline, Action, JobError
 from lava_dispatcher.logical import Boot, RetryAction
-from lava_dispatcher.actions.boot import BootAction
 from lava_dispatcher.connections.serial import ConnectDevice
 from lava_dispatcher.power import ResetDevice
 from lava_dispatcher.utils.udev import WaitDeviceBoardID
 
 
 class JLink(Boot):
-    def __init__(self, parent, parameters):
-        super().__init__(parent)
-        self.action = BootJLink()
-        self.action.section = self.action_type
-        self.action.job = self.job
-        parent.add_action(self.action, parameters)
+    @classmethod
+    def action(cls):
+        return BootJLink()
 
     @classmethod
     def accepts(cls, device, parameters):
@@ -52,17 +48,15 @@ class JLink(Boot):
         return True, "accepted"
 
 
-class BootJLink(BootAction):
+class BootJLink(RetryAction):
 
     name = "boot-jlink-image"
     description = "boot jlink image with retry"
     summary = "boot jlink image with retry"
 
     def populate(self, parameters):
-        self.internal_pipeline = Pipeline(
-            parent=self, job=self.job, parameters=parameters
-        )
-        self.internal_pipeline.add_action(BootJLinkRetry())
+        self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
+        self.pipeline.add_action(BootJLinkRetry())
 
 
 class BootJLinkRetry(RetryAction):
@@ -72,16 +66,12 @@ class BootJLinkRetry(RetryAction):
     summary = "boot jlink image"
 
     def populate(self, parameters):
-        self.internal_pipeline = Pipeline(
-            parent=self, job=self.job, parameters=parameters
-        )
+        self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
         if self.job.device.hard_reset_command:
-            self.internal_pipeline.add_action(ResetDevice())
-            self.internal_pipeline.add_action(
-                WaitDeviceBoardID(self.job.device.get("board_id"))
-            )
-        self.internal_pipeline.add_action(FlashJLinkAction())
-        self.internal_pipeline.add_action(ConnectDevice())
+            self.pipeline.add_action(ResetDevice())
+            self.pipeline.add_action(WaitDeviceBoardID(self.job.device.get("board_id")))
+        self.pipeline.add_action(FlashJLinkAction())
+        self.pipeline.add_action(ConnectDevice())
 
 
 class FlashJLinkAction(Action):
