@@ -19,24 +19,22 @@
 # with this program; if not, see <http://www.gnu.org/licenses>.
 
 import os
-from lava_common.constants import SYS_CLASS_KVM
+from lava_common.constants import DISPATCHER_DOWNLOAD_DIR, SYS_CLASS_KVM
 from lava_common.exceptions import JobError
 from lava_common.utils import debian_package_arch, debian_package_version
 from lava_dispatcher.action import Pipeline, Action
 from lava_dispatcher.logical import Boot, RetryAction
-from lava_dispatcher.actions.boot import BootAction
+from lava_dispatcher.actions.boot import BootHasMixin
 from lava_dispatcher.actions.boot.environment import ExportDeviceEnvironment
 from lava_dispatcher.shell import ExpectShellSession, ShellCommand
 from lava_dispatcher.connections.serial import QemuSession
+from lava_dispatcher.utils.docker import DockerRun
 from lava_dispatcher.utils.shell import which
 from lava_dispatcher.utils.strings import substitute
 from lava_dispatcher.utils.network import dispatcher_ip
 from lava_dispatcher.actions.boot import AutoLoginAction, OverlayUnpack
 
-# pylint: disable=too-many-instance-attributes,too-many-branches,too-many-statements
 
-
-# FIXME: decide if 'media: tmpfs' is necessary or remove from YAML. Only removable needs 'media'
 class BootQEMU(Boot):
     """
     The Boot method prepares the command to run on the dispatcher but this
@@ -49,12 +47,9 @@ class BootQEMU(Boot):
 
     compatibility = 4
 
-    def __init__(self, parent, parameters):
-        super().__init__(parent)
-        self.action = BootQEMUImageAction()
-        self.action.section = self.action_type
-        self.action.job = self.job
-        parent.add_action(self.action, parameters)
+    @classmethod
+    def action(cls):
+        return BootQEMUImageAction()
 
     @classmethod
     def accepts(cls, device, parameters):
@@ -71,24 +66,22 @@ class BootQEMU(Boot):
         return True, "accepted"
 
 
-class BootQEMUImageAction(BootAction):
+class BootQEMUImageAction(BootHasMixin, RetryAction):
 
     name = "boot-image-retry"
     description = "boot image with retry"
     summary = "boot with retry"
 
     def populate(self, parameters):
-        self.internal_pipeline = Pipeline(
-            parent=self, job=self.job, parameters=parameters
-        )
-        self.internal_pipeline.add_action(BootQemuRetry())
+        self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
+        self.pipeline.add_action(BootQemuRetry())
         if self.has_prompts(parameters):
-            self.internal_pipeline.add_action(AutoLoginAction())
+            self.pipeline.add_action(AutoLoginAction())
             if self.test_has_shell(parameters):
-                self.internal_pipeline.add_action(ExpectShellSession())
+                self.pipeline.add_action(ExpectShellSession())
                 if "transfer_overlay" in parameters:
-                    self.internal_pipeline.add_action(OverlayUnpack())
-                self.internal_pipeline.add_action(ExportDeviceEnvironment())
+                    self.pipeline.add_action(OverlayUnpack())
+                self.pipeline.add_action(ExportDeviceEnvironment())
 
 
 class BootQemuRetry(RetryAction):
@@ -98,10 +91,8 @@ class BootQemuRetry(RetryAction):
     summary = "boot QEMU image"
 
     def populate(self, parameters):
-        self.internal_pipeline = Pipeline(
-            parent=self, job=self.job, parameters=parameters
-        )
-        self.internal_pipeline.add_action(CallQemuAction())
+        self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
+        self.pipeline.add_action(CallQemuAction())
 
 
 class CallQemuAction(Action):
@@ -112,8 +103,8 @@ class CallQemuAction(Action):
 
     def __init__(self):
         super().__init__()
+        self.base_sub_command = []
         self.sub_command = []
-        self.substitutions = {}
         self.commands = []
         self.methods = None
         self.nfsrootfs = None
@@ -179,9 +170,9 @@ class CallQemuAction(Action):
             elif not boot["parameters"]["command"]:
                 self.errors = "No QEMU binary command found - missing context."
             qemu_binary = which(boot["parameters"]["command"])
-            self.sub_command = [qemu_binary]
-            self.sub_command.extend(boot["parameters"].get("options", []))
-            self.sub_command.extend(
+            self.base_sub_command = [qemu_binary]
+            self.base_sub_command.extend(boot["parameters"].get("options", []))
+            self.base_sub_command.extend(
                 ["%s" % item for item in boot["parameters"].get("extra", [])]
             )
         except AttributeError as exc:
@@ -202,21 +193,9 @@ class CallQemuAction(Action):
                 action="download-action", label=label, key="file"
             )
             if not image_arg or not action_arg:
-                self.errors = "Missing image_arg for %s. " % label
+                self.logger.warning("Missing image arg for %s", label)
                 continue
-            self.substitutions["{%s}" % label] = action_arg
             self.commands.append(image_arg)
-        self.substitutions["{NFS_SERVER_IP}"] = dispatcher_ip(
-            self.job.parameters["dispatcher"]
-        )
-        self.sub_command.extend(substitute(self.commands, self.substitutions))
-        if not self.sub_command:
-            self.errors = "No QEMU command to execute"
-        uefi_dir = self.get_namespace_data(
-            action="deployimages", label="image", key="uefi_dir"
-        )
-        if uefi_dir:
-            self.sub_command.extend(["-L", uefi_dir, "-monitor", "none"])
 
         # Check for enable-kvm command line option in device configuration.
         if method not in self.job.device["actions"]["boot"]["methods"]:
@@ -243,25 +222,54 @@ class CallQemuAction(Action):
         pexpect.spawn is one of the raw_connection objects for a Connection class.
         """
         if connection:
-            connection.finalise()
+            ns_connection = self.get_namespace_data(
+                action="shared", label="shared", key="connection", deepcopy=False
+            )
+            if connection == ns_connection:
+                connection.finalise()
+
+        self.sub_command = self.base_sub_command.copy()
+        # Generate the sub command
+        substitutions = {}
+        for label in self.get_namespace_keys("download-action"):
+            if label in ["offset", "available_loops", "uefi", "nfsrootfs"]:
+                continue
+            image_arg = self.get_namespace_data(
+                action="download-action", label=label, key="image_arg"
+            )
+            action_arg = self.get_namespace_data(
+                action="download-action", label=label, key="file"
+            )
+            if image_arg is not None:
+                substitutions["{%s}" % label] = action_arg
+        substitutions["{NFS_SERVER_IP}"] = dispatcher_ip(
+            self.job.parameters["dispatcher"], "nfs"
+        )
+        self.sub_command.extend(substitute(self.commands, substitutions))
+        uefi_dir = self.get_namespace_data(
+            action="deployimages", label="image", key="uefi_dir"
+        )
+        if uefi_dir:
+            self.sub_command.extend(["-L", uefi_dir, "-monitor", "none"])
+
         # initialise the first Connection object, a command line shell into the running QEMU.
         self.results = self.qemu_data
         guest = self.get_namespace_data(
             action="apply-overlay-guest", label="guest", key="filename"
         )
         # check for NFS
-        if "qemu-nfs" in self.methods and self.parameters.get("media") == "nfs":
+        if "qemu-nfs" == self.parameters["method"]:
             self.logger.debug("Adding NFS arguments to kernel command line.")
             root_dir = self.get_namespace_data(
                 action="extract-rootfs", label="file", key="nfsroot"
             )
-            self.substitutions["{NFSROOTFS}"] = root_dir
+            substitutions["{NFSROOTFS}"] = root_dir
             params = self.methods["qemu-nfs"]["parameters"]["append"]
             # console=ttyAMA0 root=/dev/nfs nfsroot=10.3.2.1:/var/lib/lava/dispatcher/tmp/dirname,tcp,hard,intr ip=dhcp
             append = [
                 "console=%s" % params["console"],
                 "root=/dev/nfs",
-                "%s rw" % substitute([params["nfsrootargs"]], self.substitutions)[0],
+                "%s rw" % substitute([params["nfsrootargs"]], substitutions)[0],
                 "%s" % params["ipargs"],
             ]
             self.sub_command.append("--append")
@@ -301,6 +309,18 @@ class CallQemuAction(Action):
                 value=shell_precommand_list,
             )
 
+        if "docker" in self.parameters:
+            docker = DockerRun(self.parameters["docker"]["image"])
+            docker.interactive()
+            docker.tty()
+            docker.bind_mount(DISPATCHER_DOWNLOAD_DIR)
+            docker.add_device("/dev/kvm", skip_missing=True)
+            args = []
+            if "binary" in self.parameters["docker"]:
+                args.append(self.parameters["docker"]["binary"])
+
+            self.sub_command[0] = " ".join(docker.cmdline(*args))
+
         self.logger.info("Boot command: %s", " ".join(self.sub_command))
         shell = self.shell_class(
             " ".join(self.sub_command), self.timeout, logger=self.logger
