@@ -18,10 +18,8 @@
 # along
 # with this program; if not, see <http://www.gnu.org/licenses>.
 
-import yaml
-
-from lava_dispatcher.action import Pipeline
-from lava_dispatcher.actions.deploy import DeployAction
+from lava_common.compat import yaml_safe_dump
+from lava_dispatcher.action import Action, Pipeline
 from lava_dispatcher.actions.deploy.download import DownloaderAction
 from lava_dispatcher.actions.deploy.environment import DeployDeviceEnvironment
 from lava_dispatcher.actions.deploy.overlay import OverlayAction
@@ -36,13 +34,11 @@ class FlasherRetryAction(RetryAction):
     summary = "deploy custom flasher"
 
     def populate(self, parameters):
-        self.internal_pipeline = Pipeline(
-            parent=self, job=self.job, parameters=parameters
-        )
-        self.internal_pipeline.add_action(FlasherAction())
+        self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
+        self.pipeline.add_action(FlasherAction())
 
 
-class FlasherAction(DeployAction):
+class FlasherAction(Action):
 
     name = "deploy-flasher"
     description = "deploy flasher"
@@ -61,19 +57,19 @@ class FlasherAction(DeployAction):
             self.errors = "'commands' should be a list"
 
     def populate(self, parameters):
-        self.internal_pipeline = Pipeline(
-            parent=self, job=self.job, parameters=parameters
-        )
+        self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
 
         # Download the images
         self.path = self.mkdtemp()
         for image in parameters["images"].keys():
-            self.internal_pipeline.add_action(DownloaderAction(image, self.path))
+            self.pipeline.add_action(
+                DownloaderAction(image, self.path, params=parameters["images"][image])
+            )
 
         if self.test_needs_deployment(parameters):
-            self.internal_pipeline.add_action(DeployDeviceEnvironment())
+            self.pipeline.add_action(DeployDeviceEnvironment())
         if self.test_needs_overlay(parameters):
-            self.internal_pipeline.add_action(OverlayAction())
+            self.pipeline.add_action(OverlayAction())
 
     def run(self, connection, max_end_time):
         connection = super().run(connection, max_end_time)
@@ -101,10 +97,10 @@ class FlasherAction(DeployAction):
         )
 
         # Add some device configuration
-        substitutions["{DEVICE_INFO}"] = yaml.dump(
+        substitutions["{DEVICE_INFO}"] = yaml_safe_dump(
             self.job.device.get("device_info", [])
         )
-        substitutions["{STATIC_INFO}"] = yaml.dump(
+        substitutions["{STATIC_INFO}"] = yaml_safe_dump(
             self.job.device.get("static_info", [])
         )
 
@@ -120,12 +116,9 @@ class Flasher(Deployment):
     compatibility = 4
     name = "flasher"
 
-    def __init__(self, parent, parameters):
-        super().__init__(parent)
-        self.action = FlasherRetryAction()
-        self.action.section = self.action_type
-        self.action.job = self.job
-        parent.add_action(self.action, parameters)
+    @classmethod
+    def action(cls):
+        return FlasherRetryAction()
 
     @classmethod
     def accepts(cls, device, parameters):
