@@ -19,30 +19,26 @@
 # with this program; if not, see <http://www.gnu.org/licenses>.
 
 import os
-import yaml
-from lava_dispatcher.logical import Deployment
-from lava_common.exceptions import LAVABug
+
+from lava_common.exceptions import InfrastructureError, LAVABug
 from lava_common.utils import debian_package_version
-from lava_dispatcher.action import JobError, Pipeline
-from lava_dispatcher.actions.deploy import DeployAction
+from lava_dispatcher.action import Action, JobError, Pipeline
 from lava_dispatcher.actions.deploy.overlay import OverlayAction
 from lava_dispatcher.actions.deploy.apply_overlay import ApplyLxcOverlay
 from lava_dispatcher.actions.deploy.environment import DeployDeviceEnvironment
 from lava_dispatcher.actions.boot.lxc import LxcStartAction, LxcStopAction
+from lava_dispatcher.logical import Deployment
 from lava_dispatcher.utils.shell import which
 from lava_dispatcher.protocols.lxc import LxcProtocol
 from lava_common.constants import (
     LXC_PATH,
     LXC_TEMPLATE_WITH_MIRROR,
     LXC_DEFAULT_PACKAGES,
-    UDEV_RULES_DIR,
 )
-from lava_dispatcher.utils.udev import lxc_udev_rule, lxc_udev_rule_parent
 from lava_dispatcher.utils.udev import allow_fs_label
+from lava_dispatcher_host import add_device_container_mapping
 from lava_dispatcher.utils.filesystem import lxc_path
 
-# pylint: disable=superfluous-parens,too-many-locals
-
 
 class Lxc(Deployment):
     """
@@ -53,12 +49,9 @@ class Lxc(Deployment):
     compatibility = 1
     name = "lxc"
 
-    def __init__(self, parent, parameters):
-        super().__init__(parent)
-        self.action = LxcAction()
-        self.action.section = self.action_type
-        self.action.job = self.job
-        parent.add_action(self.action, parameters)
+    @classmethod
+    def action(cls):
+        return LxcAction()
 
     @classmethod
     def accepts(cls, device, parameters):
@@ -71,7 +64,7 @@ class Lxc(Deployment):
         return False, '"lxc" was not in the device configuration deploy methods'
 
 
-class LxcAction(DeployAction):  # pylint:disable=too-many-instance-attributes
+class LxcAction(Action):
 
     name = "lxc-deploy"
     description = "download files and deploy using lxc"
@@ -84,7 +77,7 @@ class LxcAction(DeployAction):  # pylint:disable=too-many-instance-attributes
     def validate(self):
         super().validate()
         lxc_version = debian_package_version(pkg="lxc")
-        if lxc_version is not "":
+        if lxc_version != "":
             self.logger.info("lxc, installed at version: %s", lxc_version)
         else:
             self.logger.info(
@@ -100,24 +93,22 @@ class LxcAction(DeployAction):  # pylint:disable=too-many-instance-attributes
         which("lxc-create")
 
     def populate(self, parameters):
-        self.internal_pipeline = Pipeline(
-            parent=self, job=self.job, parameters=parameters
-        )
-        self.internal_pipeline.add_action(LxcCreateAction())
-        self.internal_pipeline.add_action(LxcCreateUdevRuleAction())
+        self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
+        self.pipeline.add_action(LxcCreateAction())
+        self.pipeline.add_action(LxcCreateUdevRuleAction())
         if "packages" in parameters:
-            self.internal_pipeline.add_action(LxcStartAction())
-            self.internal_pipeline.add_action(LxcAptUpdateAction())
-            self.internal_pipeline.add_action(LxcAptInstallAction())
-            self.internal_pipeline.add_action(LxcStopAction())
+            self.pipeline.add_action(LxcStartAction())
+            self.pipeline.add_action(LxcAptUpdateAction())
+            self.pipeline.add_action(LxcAptInstallAction())
+            self.pipeline.add_action(LxcStopAction())
         if self.test_needs_deployment(parameters):
-            self.internal_pipeline.add_action(DeployDeviceEnvironment())
+            self.pipeline.add_action(DeployDeviceEnvironment())
         if self.test_needs_overlay(parameters):
-            self.internal_pipeline.add_action(OverlayAction())
-            self.internal_pipeline.add_action(ApplyLxcOverlay())
+            self.pipeline.add_action(OverlayAction())
+            self.pipeline.add_action(ApplyLxcOverlay())
 
 
-class LxcCreateAction(DeployAction):
+class LxcCreateAction(Action):
     """
     Creates Lxc container.
     """
@@ -200,17 +191,23 @@ class LxcCreateAction(DeployAction):
             ]
         if self.lxc_data["lxc_arch"]:
             lxc_cmd += ["--arch", self.lxc_data["lxc_arch"]]
-        # FIXME: check if persistent name already exists and then drop allow_fail & allow_silent
-        cmd_out = self.run_command(lxc_cmd, allow_fail=True, allow_silent=True)
-        if isinstance(cmd_out, str):
-            if "exists" in cmd_out and self.lxc_data["lxc_persist"]:
-                self.logger.debug("Persistant container exists")
-                self.results = {"status": self.lxc_data["lxc_name"]}
-        elif not cmd_out:
-            raise JobError("Unable to create lxc container")
+
+        # Check if the container already exists. If this is a persistant that's
+        # ok, overwize, raise an error.
+        if not self.run_cmd(
+            ["lxc-info", "-n", self.lxc_data["lxc_name"]], allow_fail=True
+        ):
+            if not self.lxc_data["lxc_persist"]:
+                raise InfrastructureError(
+                    "lxc container %r already exists" % self.lxc_data["lxc_name"]
+                )
+            self.logger.debug("Persistant container exists")
         else:
+            # The container does not exists, just create it
+            self.run_cmd(lxc_cmd, error_msg="Unable to create lxc container")
             self.logger.debug("Container created successfully")
-            self.results = {"status": self.lxc_data["lxc_name"]}
+        self.results = {"status": self.lxc_data["lxc_name"]}
+
         # Create symlink in default container path ie., /var/lib/lxc defined by
         # LXC_PATH so that we need not add '-P' option to every lxc-* command.
         dst = os.path.join(LXC_PATH, self.lxc_data["lxc_name"])
@@ -222,7 +219,7 @@ class LxcCreateAction(DeployAction):
         return connection
 
 
-class LxcCreateUdevRuleAction(DeployAction):
+class LxcCreateUdevRuleAction(Action):
     """
     Creates Lxc related udev rules for this container.
     """
@@ -282,76 +279,26 @@ class LxcCreateUdevRuleAction(DeployAction):
             return connection
 
         device_info = self.job.device.get("device_info", [])
-        device_info_file = os.path.join(self.mkdtemp(), "device-info.yaml")
-        with open(device_info_file, "w") as device_info_obj:
-            yaml.dump(device_info, device_info_obj)
-        self.logger.debug(
-            "device info file '%s' created with:\n %s", device_info_file, device_info
-        )
-        logging_url = master_cert = slave_cert = socks_proxy = ipv6 = None
         job_id = self.job.job_id
-        # pylint: disable=no-member
-        if self.logger.handler:
-            logging_url = self.logger.handler.logging_url
-            master_cert = self.logger.handler.master_cert
-            slave_cert = self.logger.handler.slave_cert
-            socks_proxy = self.logger.handler.socks_proxy
-            ipv6 = self.logger.handler.ipv6
-            job_id = self.logger.handler.job_id
-        # The rules file will be created in job's temporary directory with
-        # the name something like '100-lava-lxc-hikey-2808.rules'
-        # where, 100 is just an arbitrary number which specifies loading
-        # priority for udevd
         job_prefix = self.job.parameters["dispatcher"].get("prefix", "")
-        rules_file_name = "100-lava-" + job_prefix + lxc_name + ".rules"
-        rules_file = os.path.join(self.mkdtemp(), rules_file_name)
-        lines = []
         for device in device_info:
             data = {
                 "serial_number": str(device.get("board_id", "")),
                 "vendor_id": device.get("usb_vendor_id"),
                 "product_id": device.get("usb_product_id"),
                 "fs_label": device.get("fs_label"),
-                "lxc_name": lxc_name,
-                "device_info_file": device_info_file,
-                "logging_url": logging_url,
-                "master_cert": master_cert,
-                "slave_cert": slave_cert,
-                "socks_proxy": socks_proxy,
-                "ipv6": ipv6,
-                "job_id": job_id,
             }
-            str_lxc_udev_rule = lxc_udev_rule(data)
-            if device.get("parent", False):
-                str_lxc_udev_rule += lxc_udev_rule_parent(data)
-            lines.append(str_lxc_udev_rule)
-        if lines:
-            self.logger.info("udev rules file '%s' created", rules_file)
-        with open(rules_file, "w") as f_obj:
-            f_obj.write("\n".join(lines))
-        for line in lines:
-            self.logger.debug(line)
-        # Create symlink to rules file inside UDEV_RULES_DIR
-        # See https://projects.linaro.org/browse/LAVA-1227
-        os.symlink(rules_file, os.path.join(UDEV_RULES_DIR, rules_file_name))
-        self.logger.info(
-            "'%s' symlinked to '%s'",
-            os.path.join(UDEV_RULES_DIR, rules_file_name),
-            rules_file,
-        )
-
-        # Reload udev rules.
-        reload_cmd = ["udevadm", "control", "--reload-rules"]
-        # FIXME: if reload fails, that is a LAVABug.
-        cmd_out = self.run_command(reload_cmd, allow_fail=True, allow_silent=True)
-        if cmd_out is False:
-            self.logger.debug("Reloading udev rules failed")
-        else:
-            self.logger.debug("udev rules reloaded.")
+            add_device_container_mapping(
+                job_prefix + job_id,
+                data,
+                lxc_name,
+                container_type="lxc",
+                logging_info=self.get_logging_info(),
+            )
         return connection
 
 
-class LxcAptUpdateAction(DeployAction):
+class LxcAptUpdateAction(Action):
     """
     apt-get update the lxc container.
     """
@@ -376,7 +323,7 @@ class LxcAptUpdateAction(DeployAction):
         return connection
 
 
-class LxcAptInstallAction(DeployAction):
+class LxcAptInstallAction(Action):
     """
     apt-get install packages to the lxc container.
     """
