@@ -21,19 +21,17 @@
 import os
 import io
 import re
-import yaml
 import base64
 import hashlib
 import tarfile
 import shutil
 
-from lava_common.compat import yaml_safe_load
+from lava_common.compat import yaml_safe_dump, yaml_safe_load
 from lava_common.decorators import nottest
 from lava_common.exceptions import InfrastructureError, JobError, LAVABug, TestError
 from lava_dispatcher.action import Action, Pipeline
-from lava_dispatcher.actions.test import TestAction
 from lava_dispatcher.utils.strings import indices
-from lava_dispatcher.utils.vcs import BzrHelper, GitHelper
+from lava_dispatcher.utils.vcs import GitHelper
 from lava_common.constants import DEFAULT_TESTDEF_NAME_CLASS, DISPATCHER_DOWNLOAD_DIR
 
 
@@ -45,10 +43,9 @@ def identify_test_definitions(test_info, namespace):
     """
     # All test definitions are deployed in each deployment - TestDefinitionAction needs to only run relevant ones.
     test_list = []
-    if namespace in test_info:
-        for test in test_info[namespace]:
-            if test["class"].needs_overlay() and ("definitions" in test["parameters"]):
-                test_list.append(test["parameters"]["definitions"])
+    for test in test_info.get(namespace, []):
+        if "definitions" in test["parameters"]:
+            test_list.append(test["parameters"]["definitions"])
     return test_list
 
 
@@ -92,7 +89,7 @@ class RepoAction(Action):
 
     @classmethod
     def select(cls, repo_type):
-        candidates = cls.__subclasses__()  # pylint: disable=no-member
+        candidates = cls.__subclasses__()
         willing = [c for c in candidates if c.accepts(repo_type)]
 
         if not willing:
@@ -237,7 +234,7 @@ class RepoAction(Action):
         self.logger.debug("uuid=%s testdef=%s", self.uuid, ret)
 
 
-class GitRepoAction(RepoAction):  # pylint: disable=too-many-public-methods
+class GitRepoAction(RepoAction):
     """
     Each repo action is for a single repository,
     tests using multiple repositories get multiple
@@ -335,80 +332,7 @@ class GitRepoAction(RepoAction):  # pylint: disable=too-many-public-methods
         return connection
 
 
-# TODO: merge with GitRepoAction
-class BzrRepoAction(RepoAction):  # pylint: disable=too-many-public-methods
-    """
-    Each repo action is for a single repository,
-    tests using multiple repositories get multiple
-    actions.
-    """
-
-    priority = 1
-    name = "bzr-repo-action"
-    description = "apply bazaar repository of tests to the test image"
-    summary = "branch a bzr test repo"
-
-    def __init__(self):
-        super().__init__()
-        self.testdef = None
-
-    def validate(self):
-        if "repository" not in self.parameters:
-            self.errors = "Bzr repository not specified in job definition"
-        if "path" not in self.parameters:
-            self.errors = "Path to YAML file not specified in the job definition"
-        if not self.valid:
-            return
-        self.vcs = BzrHelper(self.parameters["repository"])
-        super().validate()
-
-    @classmethod
-    def accepts(cls, repo_type):
-        return repo_type == "bzr"
-
-    def run(self, connection, max_end_time):
-        """
-        Clone the bazar repository into a directory
-        """
-        connection = super().run(connection, max_end_time)
-
-        # NOTE: the runner_path dir must remain empty until after the VCS clone, so let the VCS clone create the final dir
-        runner_path = self.get_namespace_data(
-            action="uuid", label="overlay_path", key=self.parameters["test_name"]
-        )
-
-        commit_id = self.vcs.clone(
-            runner_path, revision=self.parameters.get("revision")
-        )
-        if commit_id is None:
-            raise InfrastructureError(
-                "Unable to get test definition from %s (%s)"
-                % (self.vcs.binary, self.parameters)
-            )
-        self.results = {
-            "commit": commit_id,
-            "repository": self.parameters["repository"],
-            "path": self.parameters["path"],
-        }
-
-        # now read the YAML to create a testdef dict to retrieve metadata
-        yaml_file = os.path.join(runner_path, self.parameters["path"])
-        try:
-            with open(yaml_file, "r") as test_file:
-                self.testdef = yaml_safe_load(test_file)
-        except OSError as exc:
-            raise JobError(
-                "Unable to open test definition '%s': %s"
-                % (self.parameters["path"], str(exc))
-            )
-
-        # set testdef metadata in base class
-        self.store_testdef(self.testdef, "bzr", commit_id)
-
-        return connection
-
-
-class InlineRepoAction(RepoAction):  # pylint: disable=too-many-public-methods
+class InlineRepoAction(RepoAction):
 
     priority = 1
     name = "inline-repo-action"
@@ -450,7 +374,7 @@ class InlineRepoAction(RepoAction):  # pylint: disable=too-many-public-methods
         if yaml_dirname != "":
             os.makedirs(os.path.join(runner_path, yaml_dirname))
         with open(yaml_file, "w") as test_file:
-            data = yaml.safe_dump(testdef)
+            data = yaml_safe_dump(testdef)
             sha1.update(data.encode("utf-8"))
             test_file.write(data)
 
@@ -459,7 +383,7 @@ class InlineRepoAction(RepoAction):  # pylint: disable=too-many-public-methods
         return connection
 
 
-class TarRepoAction(RepoAction):  # pylint: disable=too-many-public-methods
+class TarRepoAction(RepoAction):
 
     priority = 0  # FIXME: increase priority once this is working
     name = "tar-repo-action"
@@ -514,7 +438,7 @@ class TarRepoAction(RepoAction):  # pylint: disable=too-many-public-methods
         return connection
 
 
-class UrlRepoAction(RepoAction):  # pylint: disable=too-many-public-methods
+class UrlRepoAction(RepoAction):
 
     priority = 0  # FIXME: increase priority once this is working
     name = "url-repo-action"
@@ -569,7 +493,7 @@ class UrlRepoAction(RepoAction):  # pylint: disable=too-many-public-methods
 
 
 @nottest
-class TestDefinitionAction(TestAction):
+class TestDefinitionAction(Action):
 
     name = "test-definition"
     description = "load test definitions into image"
@@ -598,9 +522,7 @@ class TestDefinitionAction(TestAction):
         install:deps script and the main run script have custom Actions.
         """
         index = []
-        self.internal_pipeline = Pipeline(
-            parent=self, job=self.job, parameters=parameters
-        )
+        self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
         self.test_list = identify_test_definitions(
             self.job.test_info, parameters["namespace"]
         )
@@ -626,7 +548,7 @@ class TestDefinitionAction(TestAction):
                     len(index),
                     handler.parameters["name"],
                 )
-                self.internal_pipeline.add_action(handler)
+                self.pipeline.add_action(handler)
                 # a genuinely unique ID based on the *database* JobID and
                 # pipeline level for reproducibility and tracking -
                 # {DB-JobID}_{PipelineLevel}, e.g. 15432.0_3.5.4
@@ -658,9 +580,9 @@ class TestDefinitionAction(TestAction):
                 index.append(handler.parameters["name"])
 
                 # add overlay handlers to the pipeline
-                self.internal_pipeline.add_action(overlay)
-                self.internal_pipeline.add_action(installer)
-                self.internal_pipeline.add_action(runsh)
+                self.pipeline.add_action(overlay)
+                self.pipeline.add_action(installer)
+                self.pipeline.add_action(runsh)
                 self.set_namespace_data(
                     action="test-definition",
                     label="test-definition",
@@ -750,7 +672,7 @@ class TestDefinitionAction(TestAction):
             with open(
                 "%s/%s/lava-test-runner.conf" % (overlay_base, stage), "a"
             ) as runner_conf:
-                for handler in self.internal_pipeline.actions:
+                for handler in self.pipeline.actions:
                     if isinstance(handler, RepoAction) and handler.stage == stage:
                         self.logger.debug("- %s", handler.parameters["test_name"])
                         runner_conf.write(handler.runner)
@@ -759,7 +681,7 @@ class TestDefinitionAction(TestAction):
 
 
 @nottest
-class TestOverlayAction(TestAction):  # pylint: disable=too-many-instance-attributes
+class TestOverlayAction(Action):
 
     name = "test-overlay"
     description = "overlay test support files onto image"
@@ -845,7 +767,7 @@ class TestOverlayAction(TestAction):  # pylint: disable=too-many-instance-attrib
 
         # FIXME: change lava-test-runner to accept a variable instead of duplicating the YAML?
         with open("%s/testdef.yaml" % runner_path, "w") as run_file:
-            yaml.safe_dump(testdef, run_file)
+            yaml_safe_dump(testdef, run_file)
 
         # write out the UUID of each test definition.
         # FIXME: is this necessary any longer?
@@ -857,7 +779,7 @@ class TestOverlayAction(TestAction):  # pylint: disable=too-many-instance-attrib
             content = self.get_namespace_data(
                 action="test", label=self.test_uuid, key="testdef_metadata"
             )
-            metadata.write(yaml.safe_dump(content))
+            metadata.write(yaml_safe_dump(content))
 
         # Need actions for the run.sh script (calling parameter support in base class)
         # and install script (also calling parameter support here.)
@@ -889,7 +811,7 @@ class TestInstallAction(TestOverlayAction):
         exist and then it will append to it.
         TestOverlayAction will then add TestInstallAction to an
         internal pipeline followed by TestParameterAction then
-        run the internal_pipeline at the start of the TestOverlayAction
+        run the pipeline at the start of the TestOverlayAction
         run step.
         """
         super().__init__()
@@ -989,7 +911,7 @@ class TestInstallAction(TestOverlayAction):
             if commit_id is None:
                 raise JobError("Unable to clone %s" % str((repo)))
 
-    def run(self, connection, max_end_time):  # pylint: disable=too-many-statements
+    def run(self, connection, max_end_time):
         connection = super().run(connection, max_end_time)
         runner_path = self.get_namespace_data(
             action="uuid", label="overlay_path", key=self.parameters["test_name"]
