@@ -20,21 +20,23 @@
 
 import re
 import time
-import yaml
 import decimal
 import logging
 import pexpect
-from collections import OrderedDict
 
+from lava_common.compat import yaml_safe_dump
 from lava_common.decorators import nottest
-from lava_common.exceptions import JobError, InfrastructureError, TestError, LAVABug
-from lava_dispatcher.actions.test import TestAction
-from lava_dispatcher.action import Pipeline
+from lava_common.exceptions import (
+    JobError,
+    InfrastructureError,
+    TestError,
+    LAVABug,
+    LAVATimeoutError,
+)
+from lava_dispatcher.action import Action, Pipeline
 from lava_dispatcher.logical import LavaTest, RetryAction
 from lava_dispatcher.connection import SignalMatch
 
-# pylint: disable=too-many-branches,too-many-statements,too-many-instance-attributes,logging-not-lazy
-
 
 def handle_testcase(params):
     data = {}
@@ -54,30 +56,27 @@ class TestShell(LavaTest):
     LavaTestShell Strategy object
     """
 
-    def __init__(self, parent, parameters):
-        super().__init__(parent)
-        self.action = TestShellRetry()
-        self.action.job = self.job
-        self.action.section = self.action_type
-        parent.add_action(self.action, parameters)
+    @classmethod
+    def action(cls, parameters):
+        return TestShellRetry()
 
     @classmethod
-    def accepts(cls, device, parameters):  # pylint: disable=unused-argument
-        if ("definition" in parameters) or ("definitions" in parameters):
+    def accepts(cls, device, parameters):
+        if "definitions" in parameters:
             return True, "accepted"
-        return False, '"definition" or "definitions" not in parameters'
+        return False, '"definitions" not in parameters'
 
     @classmethod
-    def needs_deployment_data(cls):
+    def needs_deployment_data(cls, parameters):
         """ Some, not all, deployments will want deployment_data """
         return True
 
     @classmethod
-    def needs_overlay(cls):
+    def needs_overlay(cls, parameters):
         return True
 
     @classmethod
-    def has_shell(cls):
+    def has_shell(cls, parameters):
         return True
 
 
@@ -89,10 +88,8 @@ class TestShellRetry(RetryAction):
     summary = "Retry support for Lava Test Shell"
 
     def populate(self, parameters):
-        self.internal_pipeline = Pipeline(
-            parent=self, job=self.job, parameters=parameters
-        )
-        self.internal_pipeline.add_action(TestShellAction())
+        self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
+        self.pipeline.add_action(TestShellAction())
 
 
 # FIXME: move to utils and call inside the overlay
@@ -154,7 +151,7 @@ class PatternFixup:
 
 
 @nottest
-class TestShellAction(TestAction):
+class TestShellAction(Action):
     """
     Sets up and runs the LAVA Test Shell Definition scripts.
     Supports a pre-command-list of operations necessary on the
@@ -164,6 +161,7 @@ class TestShellAction(TestAction):
     name = "lava-test-shell"
     description = "Executing lava-test-runner"
     summary = "Lava Test Shell"
+    timeout_exception = LAVATimeoutError
 
     def __init__(self):
         super().__init__()
@@ -192,14 +190,16 @@ class TestShellAction(TestAction):
         self.pattern = PatternFixup(testdef=None, count=0)
 
     def validate(self):
-        if "definitions" in self.parameters:
-            for testdef in self.parameters["definitions"]:
-                if "repository" not in testdef:
-                    self.errors = "Repository missing from test definition"
+        if "definitions" not in self.parameters:
+            raise JobError("Missing test 'definitions'")
+
+        for testdef in self.parameters["definitions"]:
+            if "repository" not in testdef:
+                self.errors = "Repository missing from test definition"
         self._reset_patterns()
         super().validate()
 
-    def run(self, connection, max_end_time):  # pylint: disable=too-many-locals
+    def run(self, connection, max_end_time):
         """
         Common run function for subclasses which define custom patterns
         """
@@ -301,9 +301,7 @@ class TestShellAction(TestAction):
 
         try:
             feedbacks = []
-            for feedback_ns in self.data.keys():  # pylint: disable=no-member
-                if feedback_ns == self.parameters.get("namespace"):
-                    continue
+            for feedback_ns in self.data.keys():
                 feedback_connection = self.get_namespace_data(
                     action="shared",
                     label="shared",
@@ -311,6 +309,8 @@ class TestShellAction(TestAction):
                     deepcopy=False,
                     parameters={"namespace": feedback_ns},
                 )
+                if feedback_connection == connection:
+                    continue
                 if feedback_connection:
                     self.logger.debug(
                         "Will listen to feedbacks from '%s' for 1 second", feedback_ns
@@ -358,12 +358,12 @@ class TestShellAction(TestAction):
             if self.current_run is not None:
                 self.logger.error("Marking unfinished test run as failed")
                 self.current_run["duration"] = "%.02f" % (time.time() - self.start)
-                self.logger.results(self.current_run)  # pylint: disable=no-member
+                self.logger.results(self.current_run)
                 self.current_run = None
 
         # Only print if the report is not empty
         if self.report:
-            self.logger.debug(yaml.dump(self.report, default_flow_style=False))
+            self.logger.debug(yaml_safe_dump(self.report, default_flow_style=False))
         if self.errors:
             raise TestError(self.errors)
         return connection
@@ -461,7 +461,7 @@ class TestShellAction(TestAction):
         if commit_id:
             res["commit_id"] = commit_id
 
-        self.logger.results(res)  # pylint: disable=no-member
+        self.logger.results(res)
         self.start = None
 
     @nottest
@@ -482,24 +482,6 @@ class TestShellAction(TestAction):
             self.logger.error(str(exc))
             return True
 
-        p_res = self.get_namespace_data(
-            action="test", label=self.signal_director.test_uuid, key="results"
-        )
-        if not p_res:
-            p_res = OrderedDict()
-            self.set_namespace_data(
-                action="test",
-                label=self.signal_director.test_uuid,
-                key="results",
-                value=p_res,
-            )
-
-        # prevent losing data in the update
-        # FIXME: support parameters and retries
-        if res["test_case_id"] in p_res:
-            raise JobError(
-                "Duplicate test_case_id in results: %s" % res["test_case_id"]
-            )
         # turn the result dict inside out to get the unique
         # test_case_id/testset_name as key and result as value
         res_data = {
@@ -526,7 +508,7 @@ class TestShellAction(TestAction):
         else:
             self.report[res["test_case_id"]] = res["result"]
         # Send the results back
-        self.logger.results(res_data)  # pylint: disable=no-member
+        self.logger.results(res_data)
 
     @nottest
     def signal_test_reference(self, params):
@@ -540,7 +522,7 @@ class TestShellAction(TestAction):
         }
         if self.testset_name:
             res_dict.update({"set": self.testset_name})
-        self.logger.results(res_dict)  # pylint: disable=no-member
+        self.logger.results(res_dict)
 
     @nottest
     def signal_test_feedback(self, params):
@@ -617,13 +599,11 @@ class TestShellAction(TestAction):
                 if "units" in res:
                     res_data["units"] = res["units"]
 
-            self.logger.results(res_data)  # pylint: disable=no-member
+            self.logger.results(res_data)
             self.report[res["test_case_id"]] = res["result"]
         return True
 
-    def check_patterns(
-        self, event, test_connection, check_char
-    ):  # pylint: disable=unused-argument
+    def check_patterns(self, event, test_connection, check_char):
         """
         Defines the base set of pattern responses.
         Stores the results of testcases inside the TestAction
