@@ -24,9 +24,6 @@ from lava_dispatcher.action import Action
 from lava_common.exceptions import JobError, InfrastructureError
 from lava_dispatcher.shell import ShellCommand, ShellSession
 
-# pylint: disable=too-many-public-methods,too-many-instance-attributes
-# pylint: disable=too-many-branches
-
 
 class ConnectDevice(Action):
     """
@@ -43,8 +40,8 @@ class ConnectDevice(Action):
     def __init__(self):
         super().__init__()
         self.session_class = (
-            ShellSession
-        )  # wraps the pexpect and provides prompt_str access
+            ShellSession  # wraps the pexpect and provides prompt_str access
+        )
         self.shell_class = ShellCommand  # runs the command to initiate the connection
         self.command = ""
         self.hardware = None
@@ -164,7 +161,7 @@ class ConnectDevice(Action):
             window=self.job.device.get_constant("spawn_maxread"),
         )
         if shell.exitstatus:
-            raise JobError(
+            raise InfrastructureError(
                 "%s command exited %d: %s"
                 % (self.command, shell.exitstatus, shell.readlines())
             )
@@ -201,8 +198,8 @@ class ConnectShell(ConnectDevice):
         )
         self.message = "Connecting to shell using"
         self.session_class = (
-            ShellSession
-        )  # wraps the pexpect and provides prompt_str access
+            ShellSession  # wraps the pexpect and provides prompt_str access
+        )
         self.shell_class = ShellCommand  # runs the command to initiate the connection
 
     def validate(self):
