@@ -47,6 +47,10 @@ class PipelineDevice(dict):
     def hard_reset_command(self):
         return self.get("commands", {}).get("hard_reset", "")
 
+    @property
+    def soft_reboot_command(self):
+        return self.get("commands", {}).get("soft_reboot", "")
+
     @property
     def soft_reset_command(self):
         return self.get("commands", {}).get("soft_reset", "")
@@ -128,6 +132,7 @@ class NewDevice(PipelineDevice):
             raise ConfigurationError("%s could not be parsed" % target)
 
         self.setdefault("power_state", "off")  # assume power is off at start of job
+        self.setdefault("dynamic_data", {})
 
     def check_config(self, job):
         """
