@@ -28,23 +28,20 @@ import os
 import subprocess  # nosec - internal use.
 import tarfile
 
-from lava_common.exceptions import (
-    InfrastructureError,
-    JobError
-)
+from lava_common.exceptions import InfrastructureError, JobError
 
 from lava_dispatcher.utils.contextmanager import chdir
 from lava_dispatcher.utils.shell import which
 
 
 # https://www.kernel.org/doc/Documentation/xz.txt
-compress_command_map = {'xz': ['xz', '--check=crc32'],  # pylint: disable=invalid-name
-                        'gz': ['gzip'],
-                        'bz2': ['bzip2']}
-decompress_command_map = {'xz': ['unxz'],  # pylint: disable=invalid-name
-                          'gz': ['gunzip'],
-                          'bz2': ['bunzip2'],
-                          'zip': ['unzip']}
+compress_command_map = {"xz": ["xz", "--check=crc32"], "gz": ["gzip"], "bz2": ["bzip2"]}
+decompress_command_map = {
+    "xz": ["unxz"],
+    "gz": ["gunzip"],
+    "bz2": ["bunzip2"],
+    "zip": ["unzip"],
+}
 
 
 def compress_file(infile, compression):
@@ -64,7 +61,7 @@ def compress_file(infile, compression):
             subprocess.check_output(cmd)  # nosec - internal use.
             return "%s.%s" % (infile, compression)
         except (OSError, subprocess.CalledProcessError) as exc:
-            raise InfrastructureError('unable to compress file %s: %s' % (infile, exc))
+            raise InfrastructureError("unable to compress file %s: %s" % (infile, exc))
 
 
 def decompress_file(infile, compression):
@@ -82,26 +79,59 @@ def decompress_file(infile, compression):
         cmd.append(infile)
         outfile = infile
         if infile.endswith(compression):
-            outfile = infile[:-(len(compression) + 1)]
+            outfile = infile[: -(len(compression) + 1)]
         try:
             subprocess.check_output(cmd)  # nosec - internal use.
             return outfile
         except (OSError, subprocess.CalledProcessError) as exc:
-            raise InfrastructureError('unable to decompress file %s: %s' % (infile, exc))
+            raise InfrastructureError(
+                "unable to decompress file %s: %s" % (infile, exc)
+            )
 
 
-def untar_file(infile, outdir, member=None, outfile=None):
+def untar_file(infile, outdir):
     try:
-        tar = tarfile.open(infile)
-        if member:
-            file_obj = tar.extractfile(member)
-            target = open(outfile, 'wb')
-            target.write(file_obj.read())
-            target.close()
-            file_obj.close()
-            tar.close()
-        else:
+        with tarfile.open(infile, encoding="utf-8") as tar:
             tar.extractall(outdir)
-            tar.close()
     except tarfile.TarError as exc:
         raise JobError("Unable to unpack %s: %s" % (infile, str(exc)))
+
+
+def cpio(directory, filename):
+    which("cpio")
+    which("find")
+    with chdir(directory):
+        try:
+            find = subprocess.check_output(
+                ["find", "."], stderr=subprocess.STDOUT
+            )  # nosec
+            return subprocess.check_output(  # nosec
+                ["cpio", "--create", "--format", "newc", "--file", filename],
+                input=find,
+                stderr=subprocess.STDOUT,
+            ).decode("utf-8", errors="replace")
+        except Exception as exc:
+            raise InfrastructureError(
+                "Unable to create cpio archive %r: %s" % (filename, exc)
+            )
+
+
+def uncpio(filename, directory):
+    which("cpio")
+    with chdir(directory):
+        try:
+            subprocess.check_output(  # nosec
+                [
+                    "cpio",
+                    "--extract",
+                    "--make-directories",
+                    "--unconditional",
+                    "--file",
+                    filename,
+                ],
+                stderr=subprocess.STDOUT,
+            )
+        except subprocess.SubprocessError as exc:
+            raise InfrastructureError(
+                "Unable to extract cpio archive %r: %s" % (filename, exc)
+            )
