@@ -31,13 +31,13 @@ from linaro_django_xmlrpc.models import AuthToken
 
 
 def help_max_length(max_length):
-    return ungettext_lazy(  # pylint: disable=no-member
+    return ungettext_lazy(
         u"Maximum length: {0} character", u"Maximum length: {0} characters", max_length
     ).format(max_length)
 
 
-class StreamEcho:  # pylint: disable=too-few-public-methods
-    def write(self, value):  # pylint: disable=no-self-use,
+class StreamEcho:
+    def write(self, value):
         return value
 
 
@@ -51,6 +51,9 @@ def description_filename(job):
 try:
     from yaml import FullLoader as Loader
 except ImportError:
+    import warnings
+
+    warnings.warn("Using unsafe yaml.Loader", DeprecationWarning)
     from yaml import Loader
 
 
@@ -113,8 +116,7 @@ def description_data(job):
     try:
         data = yaml.load(open(filename, "r"), Loader=V2Loader)
     except yaml.YAMLError as exc:
-        logger.error("Unable to parse description for %s", job.id)
-        logger.exception(exc)
+        logger.warning("Unable to parse description for %s", job.id)
     except OSError as exc:
         logger.error("Unable to open description for %s", job.id)
         logger.exception(exc)
