@@ -41,7 +41,7 @@ from django.http.response import HttpResponse, StreamingHttpResponse
 from django.views.decorators.http import require_POST
 from django.shortcuts import render, loader
 
-from lava_common.compat import yaml_load
+from lava_common.compat import yaml_dump, yaml_load
 from lava_server.views import index as lava_index
 from lava_server.bread_crumbs import BreadCrumb, BreadCrumbTrail
 from django.shortcuts import get_object_or_404
@@ -73,8 +73,6 @@ from lava_scheduler_app.views import get_restricted_job
 
 from lava_server.lavatable import LavaView
 
-# pylint: disable=too-many-ancestors,invalid-name
-
 
 class ResultsView(LavaView):
     """
@@ -84,8 +82,12 @@ class ResultsView(LavaView):
     def get_queryset(self):
         return (
             TestSuite.objects.all()
-            .select_related("job")
-            .prefetch_related("job__actual_device", "job__actual_device__device_type")
+            .select_related(
+                "job",
+                "job__actual_device",
+                "job__actual_device__device_type",
+                "job__submitter",
+            )
             .order_by("-job__id", "name")
         )
 
@@ -139,9 +141,8 @@ def testjob(request, job):
     failed_definitions = []
     yaml_dict = OrderedDict()
 
-    testdata = TestData.objects.filter(testjob=job).first()
-    if testdata:
-        for data in testdata.attributes.all().order_by("name"):
+    if hasattr(job, "testdata"):
+        for data in job.testdata.attributes.all().order_by("name"):
             yaml_dict[str(data.name)] = str(data.value)
 
     RequestConfig(request, paginate={"per_page": suite_table.length}).configure(
@@ -205,7 +206,7 @@ def testjob_yaml(request, job):
     def test_case_stream():
         for test_suite in suites:
             for test_case in test_suite.testcase_set.all():
-                yield yaml.dump([export_testcase(test_case)], Dumper=yaml.CDumper)
+                yield yaml_dump([export_testcase(test_case)])
 
     response = StreamingHttpResponse(test_case_stream(), content_type="text/yaml")
     filename = "lava_%s.yaml" % job.id
@@ -224,7 +225,7 @@ def testjob_yaml_summary(request, job):
     yaml_list = []
     for test_suite in suites:
         yaml_list.append(export_testsuite(test_suite))
-    yaml.dump(yaml_list, response, Dumper=yaml.CDumper)
+    yaml_dump(yaml_list, response)
     return response
 
 
@@ -330,7 +331,7 @@ def suite_yaml(request, job, pk):
     testcases = get_testcases_with_limit(test_suite, limit, offset)
     for test_case in testcases:
         yaml_list.append(export_testcase(test_case))
-    yaml.dump(yaml_list, response, Dumper=yaml.CDumper)
+    yaml_dump(yaml_list, response)
     return response
 
 
@@ -352,17 +353,16 @@ def metadata_export(request, job):
     job = get_object_or_404(TestJob, pk=job)
     check_request_auth(request, job)
     # testdata from job & export
-    testdata = TestData.objects.filter(testjob=job).first()
-    if not testdata:
-        raise Http404("No TestData matches the given query.")
+    if not hasattr(job, "testdata"):
+        raise Http404("No TestData present in test job.")
     response = HttpResponse(content_type="text/yaml")
     filename = "lava_metadata_%s.yaml" % job.id
     response["Content-Disposition"] = 'attachment; filename="%s"' % filename
     yaml_dict = {}
     # hide internal python objects
-    for data in testdata.attributes.all():
+    for data in job.testdata.attributes.all():
         yaml_dict[str(data.name)] = str(data.value)
-    yaml.dump(yaml_dict, response)
+    yaml_dump(yaml_dict, response)
     return response
 
 
@@ -482,9 +482,7 @@ def testcase_yaml(request, pk):
     response = HttpResponse(content_type="text/yaml")
     filename = "lava_%s.yaml" % testcase.name
     response["Content-Disposition"] = 'attachment; filename="%s"' % filename
-    yaml.dump(
-        export_testcase(testcase, with_buglinks=True), response, Dumper=yaml.CDumper
-    )
+    yaml_dump(export_testcase(testcase, with_buglinks=True), response)
     return response
 
 
