@@ -26,6 +26,7 @@ from django.contrib import admin
 from django.contrib.auth.admin import UserAdmin, GroupAdmin
 from django.contrib.auth.models import Permission, Group
 from django.db import transaction
+from django.db.models import Prefetch, Q
 from django.conf import settings
 
 from lava_scheduler_app.models import (
@@ -37,6 +38,7 @@ from lava_scheduler_app.models import (
     DeviceType,
     GroupDeviceTypePermission,
     GroupDevicePermission,
+    GroupWorkerPermission,
     JobFailureTag,
     NotificationRecipient,
     ProcessorFamily,
@@ -48,13 +50,14 @@ from lava_scheduler_app.models import (
 from linaro_django_xmlrpc.models import AuthToken
 
 # django admin API itself isn't pylint clean, so some settings must be suppressed.
-# pylint: disable=no-self-use,function-redefined
 
 
 class GroupObjectPermissionInline(admin.TabularInline):
     extra = 0
     supported_permissions = (
-        DeviceType.PERMISSIONS_PRIORITY + Device.PERMISSIONS_PRIORITY
+        DeviceType.PERMISSIONS_PRIORITY
+        + Device.PERMISSIONS_PRIORITY
+        + Worker.PERMISSIONS_PRIORITY
     )
 
     def formfield_for_foreignkey(self, db_field, request=None, **kwargs):
@@ -79,6 +82,11 @@ class GroupDevicePermissionInline(GroupObjectPermissionInline):
     extra = 0
 
 
+class GroupWorkerPermissionInline(GroupObjectPermissionInline):
+    model = GroupWorkerPermission
+    extra = 0
+
+
 class AliasAdmin(admin.ModelAdmin):
     def get_readonly_fields(self, _, obj=None):
         if obj:  # editing an existing object
@@ -109,9 +117,7 @@ class CoreAdmin(admin.ModelAdmin):
         return self.readonly_fields
 
 
-def expire_user_action(
-    modeladmin, request, queryset
-):  # pylint: disable=unused-argument
+def expire_user_action(modeladmin, request, queryset):
     for user in queryset.filter(is_active=True):
         AuthToken.objects.filter(user=user).delete()
         user.is_staff = False
@@ -168,7 +174,7 @@ admin.site.unregister(Group)
 admin.site.register(Group, CustomGroupAdmin)
 
 
-def cancel_action(modeladmin, request, queryset):  # pylint: disable=unused-argument
+def cancel_action(modeladmin, request, queryset):
     with transaction.atomic():
         for testjob in queryset:
             # TODO: hacky. until django 2.0+ is used and select_for_update gets
@@ -337,6 +343,13 @@ class DeviceAdmin(admin.ModelAdmin):
             super(DeviceAdmin, self)
             .get_queryset(request)
             .select_related("worker_host", "device_type")
+            .prefetch_related(
+                Prefetch(
+                    "testjobs",
+                    queryset=TestJob.objects.filter(~Q(state=TestJob.STATE_FINISHED)),
+                    to_attr="running_jobs",
+                )
+            )
         )
 
     def has_health_check(self, obj):
@@ -460,9 +473,7 @@ class TestJobAdmin(admin.ModelAdmin):
     ordering = ["-submit_time"]
 
 
-def disable_health_check_action(
-    modeladmin, request, queryset
-):  # pylint: disable=unused-argument
+def disable_health_check_action(modeladmin, request, queryset):
     queryset.update(disable_health_check=False)
 
 
@@ -607,6 +618,7 @@ class WorkerAdmin(admin.ModelAdmin):
     readonly_fields = ("state",)
     ordering = ["hostname"]
     actions = [worker_health_active, worker_health_maintenance, worker_health_retired]
+    inlines = [GroupWorkerPermissionInline]
 
     def get_readonly_fields(self, _, obj=None):
         if obj:  # editing an existing object
