@@ -18,28 +18,25 @@
 # along with LAVA.  If not, see <http://www.gnu.org/licenses/>.
 
 import contextlib
-import glob
 import os
 import xmlrpc.client
 
-from django.conf import settings
 from django.db import IntegrityError
 from django.forms import ValidationError
 
 from linaro_django_xmlrpc.models import ExposedV2API
 from lava_scheduler_app.api import check_perm
 from lava_scheduler_app.models import Alias, DeviceType
+from lava_server.files import File
 
 
 class SchedulerDeviceTypesAPI(ExposedV2API):
     def _available_device_types(self):
         """ List avaiable device types by looking at the configuration files """
         available_types = []
-        pattern = os.path.join(settings.DEVICE_TYPES_PATH, "*.jinja2")
-        for fname in glob.iglob(pattern):
-            device_type = os.path.basename(fname[:-7])
+        for device_type in File("device-type").list("*.jinja2"):
             if not device_type.startswith("base"):
-                available_types.append(device_type)
+                available_types.append(device_type[:-7])
         available_types.sort()
         return available_types
 
@@ -140,12 +137,9 @@ class SchedulerDeviceTypesAPI(ExposedV2API):
             raise xmlrpc.client.Fault(400, "Invalid device-type '%s'" % name)
 
         try:
-            filename = os.path.join(
-                "/etc/lava-server/dispatcher-config/health-checks", name
+            return xmlrpc.client.Binary(
+                File("health-check", name).read().encode("utf-8")
             )
-            filename += ".yaml" if not filename.endswith(".yaml") else ""
-            with open(filename, "r") as f_in:
-                return xmlrpc.client.Binary(f_in.read().encode("utf-8"))
         except FileNotFoundError:
             raise xmlrpc.client.Fault(404, "Device-type '%s' was not found." % name)
         except OSError as exc:
@@ -188,10 +182,9 @@ class SchedulerDeviceTypesAPI(ExposedV2API):
             raise xmlrpc.client.Fault(400, "Invalid device-type '%s'" % name)
 
         try:
-            filename = os.path.join(settings.DEVICE_TYPES_PATH, name)
-            filename += ".jinja2" if not filename.endswith(".jinja2") else ""
-            with open(filename, "r") as f_in:
-                return xmlrpc.client.Binary(f_in.read().encode("utf-8"))
+            return xmlrpc.client.Binary(
+                File("device-type", name).read().encode("utf-8")
+            )
         except FileNotFoundError:
             raise xmlrpc.client.Fault(404, "Device-type '%s' was not found." % name)
         except OSError as exc:
@@ -233,12 +226,7 @@ class SchedulerDeviceTypesAPI(ExposedV2API):
             raise xmlrpc.client.Fault(400, "Invalid device-type '%s'" % name)
 
         try:
-            filename = os.path.join(
-                "/etc/lava-server/dispatcher-config/health-checks", name
-            )
-            filename += ".yaml" if not filename.endswith(".yaml") else ""
-            with open(filename, "w") as f_out:
-                f_out.write(config)
+            File("health-check", name).write(config)
         except OSError as exc:
             raise xmlrpc.client.Fault(
                 400, "Unable to write health-check: %s" % exc.strerror
@@ -278,10 +266,7 @@ class SchedulerDeviceTypesAPI(ExposedV2API):
             raise xmlrpc.client.Fault(400, "Invalid device-type '%s'" % name)
 
         try:
-            filename = os.path.join(settings.DEVICE_TYPES_PATH, name)
-            filename += ".jinja2" if not filename.endswith(".jinja2") else ""
-            with open(filename, "w") as f_out:
-                f_out.write(config)
+            File("device-type", name).write(config)
         except OSError as exc:
             raise xmlrpc.client.Fault(
                 400, "Unable to write device-type configuration: %s" % exc.strerror
@@ -313,6 +298,7 @@ class SchedulerDeviceTypesAPI(ExposedV2API):
             dt
             for dt in DeviceType.objects.all()
             .visible_by_user(self.user)
+            .prefetch_related("device_set")
             .order_by("name")
         ]
         ret = []
