@@ -18,18 +18,19 @@
 # along with LAVA.  If not, see <http://www.gnu.org/licenses/>.
 
 import os
-from datetime import datetime, timedelta
+from datetime import timedelta
 import voluptuous
 import xmlrpc.client
-import yaml
 
 from django.conf import settings
+from django.utils import timezone
 
 from linaro_django_xmlrpc.models import ExposedV2API
 
+from lava_common.compat import yaml_safe_load
 import lava_common.schemas as schemas
 from lava_scheduler_app.api import SchedulerAPI
-from lava_scheduler_app.logutils import read_logs
+from lava_scheduler_app.logutils import logs_instance
 from lava_scheduler_app.models import TestJob
 from lava_results_app.models import TestCase
 
@@ -211,7 +212,7 @@ class SchedulerJobsAPI(ExposedV2API):
 
         # since
         if since:
-            end_time = datetime.now()
+            end_time = timezone.now()
             # search back in time
             start_time = end_time - timedelta(minutes=since)
             jobs = jobs.filter(end_time__range=[start_time, end_time])
@@ -354,7 +355,7 @@ class SchedulerJobsAPI(ExposedV2API):
         job_finished = job.state == TestJob.STATE_FINISHED
 
         try:
-            data = read_logs(job.output_dir, start, end)
+            data = logs_instance.read(job, start, end)
             return (job_finished, xmlrpc.client.Binary(data.encode("utf-8")))
         except OSError:
             return (job_finished, xmlrpc.client.Binary("[]".encode("utf-8")))
@@ -494,7 +495,7 @@ class SchedulerJobsAPI(ExposedV2API):
         This function returns None if the job definition is valid. Returns a
         dictionary in case of error with the key and msg.
         """
-        data = yaml.safe_load(definition)
+        data = yaml_safe_load(definition)
         try:
             schemas.validate(
                 data,
