@@ -21,11 +21,13 @@ import contextlib
 import pathlib
 import xmlrpc.client
 
+from django.conf import settings
 from django.db import IntegrityError, transaction
 
 from linaro_django_xmlrpc.models import ExposedV2API
 from lava_scheduler_app.api import check_perm
 from lava_scheduler_app.models import Worker
+from lava_server.files import File
 
 
 class SchedulerWorkersAPI(ExposedV2API):
@@ -94,18 +96,18 @@ class SchedulerWorkersAPI(ExposedV2API):
         except Worker.DoesNotExist:
             raise xmlrpc.client.Fault(404, "Worker '%s' was not found." % hostname)
 
-        base = pathlib.Path("/etc/lava-server/dispatcher.d")
-        with contextlib.suppress(OSError):
-            data = (base / hostname / "dispatcher.yaml").read_text(encoding="utf-8")
-            return xmlrpc.client.Binary(data.encode("utf-8"))
-
-        with contextlib.suppress(OSError):
-            data = (base / ("%s.yaml" % hostname)).read_text(encoding="utf-8")
-            return xmlrpc.client.Binary(data.encode("utf-8"))
-
-        raise xmlrpc.client.Fault(
-            404, "Worker '%s' does not have a configuration" % hostname
-        )
+        try:
+            return xmlrpc.client.Binary(
+                File("dispatcher", hostname).read().encode("utf-8")
+            )
+        except FileNotFoundError:
+            raise xmlrpc.client.Fault(
+                404, "Worker '%s' does not have a dispatcher configuration" % hostname
+            )
+        except OSError as exc:
+            raise xmlrpc.client.Fault(
+                400, "Unable to read dispatcher configuration: %s" % exc.strerror
+            )
 
     def get_env(self, hostname):
         """
@@ -139,20 +141,14 @@ class SchedulerWorkersAPI(ExposedV2API):
         except Worker.DoesNotExist:
             raise xmlrpc.client.Fault(404, "Worker '%s' was not found." % hostname)
 
-        base = pathlib.Path("/etc/lava-server/")
-        with contextlib.suppress(OSError):
-            data = (base / "dispatcher.d" / hostname / "env.yaml").read_text(
-                encoding="utf-8"
+        try:
+            return xmlrpc.client.Binary(File("env", hostname).read().encode("utf-8"))
+        except FileNotFoundError:
+            raise xmlrpc.client.Fault(
+                404, "Worker '%s' does not have an env file" % hostname
             )
-            return xmlrpc.client.Binary(data.encode("utf-8"))
-
-        with contextlib.suppress(OSError):
-            data = (base / "env.yaml").read_text(encoding="utf-8")
-            return xmlrpc.client.Binary(data.encode("utf-8"))
-
-        raise xmlrpc.client.Fault(
-            404, "Worker '%s' does not have a configuration" % hostname
-        )
+        except OSError as exc:
+            raise xmlrpc.client.Fault(400, "Unable to read env file: %s" % exc.strerror)
 
     @check_perm("lava_scheduler_app.change_worker")
     def set_config(self, hostname, config):
@@ -185,12 +181,8 @@ class SchedulerWorkersAPI(ExposedV2API):
         except Worker.DoesNotExist:
             raise xmlrpc.client.Fault(404, "Worker '%s' was not found." % hostname)
 
-        path = (
-            pathlib.Path("/etc/lava-server/dispatcher.d") / hostname / "dispatcher.yaml"
-        )
         with contextlib.suppress(OSError):
-            path.parent.mkdir(mode=0o755, parents=True, exist_ok=True)
-            path.write_text(config, encoding="utf-8")
+            File("dispatcher", hostname).write(config)
             return True
 
         return False
@@ -226,10 +218,95 @@ class SchedulerWorkersAPI(ExposedV2API):
         except Worker.DoesNotExist:
             raise xmlrpc.client.Fault(404, "Worker '%s' was not found." % hostname)
 
-        path = pathlib.Path("/etc/lava-server/dispatcher.d") / hostname / "env.yaml"
+        with contextlib.suppress(OSError):
+            File("env", hostname).write(env)
+            return True
+
+        return False
+
+    def get_certificate(self, hostname):
+        """
+        Name
+        ----
+        `scheduler.workers.get_certificate` (`hostname`)
+
+        Description
+        -----------
+        Return the worker public certificate found in location specified
+        in django settings.
+
+        Arguments
+        ---------
+        `hostname`: string
+          Hostname of the worker
+
+        Return value
+        ------------
+        This function returns the worker public certificate found on master
+        """
+        # Sanitize hostname as we will use it in a path
+        if len(pathlib.Path(hostname).parts) != 1:
+            raise xmlrpc.client.Fault(400, "Invalid worker name")
+
+        # Find the worker in the database
+        try:
+            worker = Worker.objects.get(hostname=hostname)
+        except Worker.DoesNotExist:
+            raise xmlrpc.client.Fault(404, "Worker '%s' was not found." % hostname)
+
+        if not worker.can_change(self.user):
+            raise xmlrpc.client.Fault(
+                403, "Insufficient permissions. Please contact system administrator."
+            )
+
+        path = pathlib.Path(settings.SLAVES_CERTS) / ("%s.key" % worker.hostname)
+        with contextlib.suppress(OSError):
+            data = (path).read_text(encoding="utf-8")
+            return xmlrpc.client.Binary(data.encode("utf-8"))
+
+        raise xmlrpc.client.Fault(
+            404, "Worker '%s' does not have a certificate uploaded" % hostname
+        )
+
+    def set_certificate(self, hostname, key):
+        """
+        Name
+        ----
+        `scheduler.workers.set_certificate` (`hostname`, `key`)
+
+        Description
+        -----------
+        Upload worker public certificate to master.
+
+        Arguments
+        ---------
+        `hostname`: string
+          Hostname of the worker
+        `key`: string
+          The public worker certificate
+
+        Return value
+        ------------
+        True if the certificate was saved to file, False otherwise.
+        """
+        # Sanitize hostname as we will use it in a path
+        if len(pathlib.Path(hostname).parts) != 1:
+            raise xmlrpc.client.Fault(400, "Invalid worker name")
+
+        try:
+            worker = Worker.objects.get(hostname=hostname)
+        except Worker.DoesNotExist:
+            raise xmlrpc.client.Fault(404, "Worker '%s' was not found." % hostname)
+
+        if not worker.can_change(self.user):
+            raise xmlrpc.client.Fault(
+                403, "Insufficient permissions. Please contact system administrator."
+            )
+
+        path = pathlib.Path(settings.SLAVES_CERTS) / ("%s.key" % worker.hostname)
         with contextlib.suppress(OSError):
             path.parent.mkdir(mode=0o755, parents=True, exist_ok=True)
-            path.write_text(env, encoding="utf-8")
+            path.write_text(key, encoding="utf-8")
             return True
 
         return False
@@ -289,14 +366,15 @@ class SchedulerWorkersAPI(ExposedV2API):
                 d.hostname for d in worker.device_set.all().order_by("hostname")
             ],
             "last_ping": worker.last_ping,
+            "job_limit": worker.job_limit,
         }
 
     @check_perm("lava_scheduler_app.change_worker")
-    def update(self, hostname, description=None, health=None):
+    def update(self, hostname, description=None, health=None, job_limit=None):
         """
         Name
         ----
-        `scheduler.workers.update` (`hostname`, `description=None`, `health=None`)
+        `scheduler.workers.update` (`hostname`, `description=None`, `health=None`, `job_limit=None`)
 
         Description
         -----------
@@ -311,6 +389,8 @@ class SchedulerWorkersAPI(ExposedV2API):
           Description of the worker
         `health`: string
           Set worker health ("ACTIVE", "MAINTENANCE" or "RETIRED")
+        `job_limit`: positive integer
+          Set job limit for this worker
 
         Return value
         ------------
@@ -335,4 +415,9 @@ class SchedulerWorkersAPI(ExposedV2API):
                 else:
                     raise xmlrpc.client.Fault(400, "Invalid health: %s" % health)
 
+            if job_limit is not None:
+                if not isinstance(job_limit, int) or job_limit < 0:
+                    raise xmlrpc.client.fault(400, "Invalid job limit")
+                worker.job_limit = job_limit
+
             worker.save()
