@@ -18,7 +18,6 @@
 # You should have received a copy of the GNU Affero General Public License
 # along with LAVA.  If not, see <http://www.gnu.org/licenses/>.
 
-# pylint: disable=too-many-lines
 
 import contextlib
 import datetime
@@ -30,6 +29,7 @@ import gzip
 import simplejson
 import yaml
 
+from django.db import transaction
 from django.db.models import Q
 from django.conf import settings
 from django.contrib.auth.models import User, Group, Permission
@@ -37,32 +37,36 @@ from django.contrib.contenttypes.models import ContentType
 from django.contrib.admin.models import LogEntry, ADDITION, CHANGE
 from django.contrib.postgres.fields import ArrayField
 from django.contrib.sites.models import Site
-from django.core.exceptions import ImproperlyConfigured, ValidationError
+from django.core.exceptions import (
+    ImproperlyConfigured,
+    PermissionDenied,
+    ValidationError,
+)
 from django.urls import reverse
 from django.db import models
 from django.utils import timezone
 from django.utils.translation import ugettext_lazy as _
 from django.utils.safestring import mark_safe
 
-from lava_common.compat import yaml_safe_load
+from lava_common.compat import yaml_dump, yaml_safe_load, yaml_safe_dump
 from lava_common.decorators import nottest
 from lava_results_app.utils import export_testcase
 from lava_scheduler_app import utils
-from lava_scheduler_app.logutils import read_logs
+from lava_scheduler_app.logutils import logs_instance
+import lava_scheduler_app.environment as environment
 from lava_scheduler_app.managers import (
     RestrictedDeviceTypeQuerySet,
     RestrictedDeviceQuerySet,
     RestrictedTestJobQuerySet,
+    RestrictedWorkerQuerySet,
     GroupObjectPermissionManager,
 )
 from lava_scheduler_app.schema import SubmissionException, validate_device
 from lava_server.compat import add_permissions
+from lava_server.files import File
 
 import requests
 
-# pylint: disable=invalid-name,no-self-use,too-many-public-methods,too-few-public-methods
-# pylint: disable=too-many-branches,too-many-return-statements,too-many-instance-attributes
-
 
 class JSONDataError(ValueError):
     """Error raised when JSON is syntactically valid but ill-formed."""
@@ -314,7 +318,7 @@ class DeviceType(RestrictedObject):
     HEALTH_PER_HOUR = 0
     HEALTH_PER_JOB = 1
     HEALTH_DENOMINATOR = ((HEALTH_PER_HOUR, "hours"), (HEALTH_PER_JOB, "jobs"))
-
+    HEALTH_DENOMINATOR_REVERSE = {"hours": HEALTH_PER_HOUR, "jobs": HEALTH_PER_JOB}
     health_denominator = models.IntegerField(
         choices=HEALTH_DENOMINATOR,
         default=HEALTH_PER_HOUR,
@@ -352,11 +356,21 @@ class DeviceType(RestrictedObject):
         return self.is_permission_restricted(perm)
 
 
-class Worker(models.Model):
+class Worker(RestrictedObject):
     """
     A worker node to which devices are attached.
     """
 
+    CHANGE_PERMISSION = "lava_scheduler_app.change_worker"
+    VIEW_PERMISSION = "lava_scheduler_app.view_worker"
+
+    # Only change permission is supported for workers.
+    PERMISSIONS_PRIORITY = [CHANGE_PERMISSION]
+
+    objects = RestrictedWorkerQuerySet.as_manager()
+
+    MASTER_UPGRADE_NOTIFICATION = "master_upgrade_notification.txt"
+
     hostname = models.CharField(
         verbose_name=_("Hostname"),
         max_length=200,
@@ -367,6 +381,7 @@ class Worker(models.Model):
 
     STATE_ONLINE, STATE_OFFLINE = range(2)
     STATE_CHOICES = ((STATE_ONLINE, "Online"), (STATE_OFFLINE, "Offline"))
+    STATE_REVERSE = {"Online": STATE_ONLINE, "Offline": STATE_OFFLINE}
     state = models.IntegerField(
         choices=STATE_CHOICES, default=STATE_OFFLINE, editable=False
     )
@@ -377,6 +392,11 @@ class Worker(models.Model):
         (HEALTH_MAINTENANCE, "Maintenance"),
         (HEALTH_RETIRED, "Retired"),
     )
+    HEALTH_REVERSE = {
+        "Active": HEALTH_ACTIVE,
+        "Maintenance": HEALTH_MAINTENANCE,
+        "Retired": HEALTH_RETIRED,
+    }
     health = models.IntegerField(choices=HEALTH_CHOICES, default=HEALTH_ACTIVE)
 
     description = models.TextField(
@@ -390,20 +410,27 @@ class Worker(models.Model):
 
     last_ping = models.DateTimeField(verbose_name=_("Last ping"), default=timezone.now)
 
+    job_limit = models.PositiveIntegerField(default=0)
+
+    version = models.CharField(
+        verbose_name=_("Dispatcher version"),
+        max_length=50,
+        null=True,
+        default=None,
+        blank=True,
+    )
+
+    master_version_notified = models.CharField(
+        verbose_name=_("Master version notified"),
+        max_length=50,
+        null=True,
+        default=None,
+        blank=True,
+    )
+
     def __str__(self):
         return self.hostname
 
-    def can_change(self, user):
-        return user.has_perm("lava_scheduler_app.change_worker")
-
-    def can_update(self, user):
-        if user.has_perm("lava_scheduler_app.change_worker"):
-            return True
-        elif user.username == "lava-health":
-            return True
-        else:
-            return False
-
     def get_absolute_url(self):
         return reverse("lava.scheduler.worker.detail", args=[self.pk])
 
@@ -468,6 +495,14 @@ class Worker(models.Model):
             change_message=reason,
         )
 
+    def can_change(self, user):
+        if user.username == "lava-health":
+            return True
+        return user.has_perm(self.CHANGE_PERMISSION, self)
+
+    def has_any_permission_restrictions(self, perm):
+        return self.is_permission_restricted(perm)
+
 
 class Device(RestrictedObject):
     """
@@ -553,14 +588,24 @@ class Device(RestrictedObject):
         (STATE_RESERVED, "Reserved"),
         (STATE_RUNNING, "Running"),
     )
+    STATE_REVERSE = {
+        "Idle": STATE_IDLE,
+        "Reserved": STATE_RESERVED,
+        "Running": STATE_RUNNING,
+    }
     state = models.IntegerField(
         choices=STATE_CHOICES, default=STATE_IDLE, editable=False
     )
 
     # The device health helps to decide what to do next with the device
-    HEALTH_GOOD, HEALTH_UNKNOWN, HEALTH_LOOPING, HEALTH_BAD, HEALTH_MAINTENANCE, HEALTH_RETIRED = range(
-        6
-    )
+    (
+        HEALTH_GOOD,
+        HEALTH_UNKNOWN,
+        HEALTH_LOOPING,
+        HEALTH_BAD,
+        HEALTH_MAINTENANCE,
+        HEALTH_RETIRED,
+    ) = range(6)
     HEALTH_CHOICES = (
         (HEALTH_GOOD, "Good"),
         (HEALTH_UNKNOWN, "Unknown"),
@@ -606,9 +651,17 @@ class Device(RestrictedObject):
         )
 
     def current_job(self):
+        # This method will use the 'running_jobs' attribute if present which
+        # is a prefetch_related attribute containing non-finished jobs
+        # ie. Prefetch('testjobs', queryset=TestJob.objects.filter(~Q(state=TestJob.STATE_FINISHED)), to_attr='running_jobs')
         try:
-            return self.testjobs.get(~Q(state=TestJob.STATE_FINISHED))
-        except TestJob.DoesNotExist:
+            return self.running_jobs[0]
+        except AttributeError:
+            try:
+                return self.testjobs.get(~Q(state=TestJob.STATE_FINISHED))
+            except TestJob.DoesNotExist:
+                return None
+        except IndexError:
             return None
 
     def get_absolute_url(self):
@@ -805,26 +858,12 @@ class Device(RestrictedObject):
             job_ctx = {}
 
         if output_format == "raw":
-            try:
-                with open(
-                    os.path.join(settings.DEVICES_PATH, "%s.jinja2" % self.hostname),
-                    "r",
-                ) as f_in:
-                    return f_in.read()
-            except OSError:
-                return None
-
-        # Create the environment
-        env = jinja2.Environment(  # nosec - YAML, not HTML, no XSS scope.
-            autoescape=False,
-            loader=jinja2.FileSystemLoader(
-                [settings.DEVICES_PATH, settings.DEVICE_TYPES_PATH]
-            ),
-            trim_blocks=True,
-        )
+            with contextlib.suppress(OSError):
+                return File("device", self.hostname).read()
+            return None
 
         try:
-            template = env.get_template("%s.jinja2" % self.hostname)
+            template = environment.devices().get_template("%s.jinja2" % self.hostname)
             device_template = template.render(**job_ctx)
         except jinja2.TemplateError:
             return None
@@ -867,10 +906,7 @@ class Device(RestrictedObject):
 
     def save_configuration(self, data):
         try:
-            with open(
-                os.path.join(settings.DEVICES_PATH, "%s.jinja2" % self.hostname), "w"
-            ) as f_out:
-                f_out.write(data)
+            File("device", self.hostname).write(data)
             return True
         except OSError as exc:
             logger = logging.getLogger("lava_scheduler_app")
@@ -1012,8 +1048,8 @@ def _check_submit_to_devices(device_list, user):
     allow = list(device_list.accessible_by_user(user, Device.SUBMIT_PERMISSION))
     if not allow:
         raise DevicesUnavailableException(
-            "No devices from [%s] pool are currently available to user %s"
-            % (device_list.values_list("hostname", flat=True), user)
+            "No devices from %s pool are currently available to user %s"
+            % (list(device_list.values_list("hostname", flat=True)), user)
         )
     return allow
 
@@ -1070,7 +1106,6 @@ def _get_device_type(user, name):
     return device_type
 
 
-# pylint: disable=too-many-arguments,too-many-locals
 def _create_pipeline_job(
     job_data,
     user,
@@ -1114,7 +1149,7 @@ def _create_pipeline_job(
                 raise SubmissionException("Invalid job priority: %r" % key)
 
     if not orig:
-        orig = yaml.safe_dump(job_data)
+        orig = yaml_safe_dump(job_data)
 
     is_public = False
     viewing_groups = []
@@ -1133,30 +1168,29 @@ def _create_pipeline_job(
                     "No known groups were found in the visibility list."
                 )
 
-    job = TestJob(
-        definition=yaml.safe_dump(job_data),
-        original_definition=orig,
-        submitter=user,
-        requested_device_type=device_type,
-        target_group=target_group,
-        description=job_data["job_name"],
-        health_check=health_check,
-        priority=priority,
-        is_public=is_public,
-    )
-    job.save()
+    with transaction.atomic():
+        job = TestJob(
+            definition=yaml_safe_dump(job_data),
+            original_definition=orig,
+            submitter=user,
+            requested_device_type=device_type,
+            target_group=target_group,
+            description=job_data["job_name"],
+            health_check=health_check,
+            priority=priority,
+            is_public=is_public,
+        )
+        job.save()
 
-    # need a valid job (witha  primary_key )before tags and groups can be
-    # assigned
-    job.tags.add(*taglist)
-    job.viewing_groups.add(*viewing_groups)
+        # need a valid job (witha  primary_key )before tags and groups can be
+        # assigned
+        job.tags.add(*taglist)
+        job.viewing_groups.add(*viewing_groups)
 
     return job
 
 
-def _pipeline_protocols(
-    job_data, user, yaml_data=None
-):  # pylint: disable=too-many-locals,too-many-branches
+def _pipeline_protocols(job_data, user, yaml_data=None):
     """
     Handle supported pipeline protocols
     Check supplied parameters and change the device selection if necessary.
@@ -1193,7 +1227,7 @@ def _pipeline_protocols(
         return device_list
 
     if not yaml_data:
-        yaml_data = yaml.safe_dump(job_data)
+        yaml_data = yaml_safe_dump(job_data)
     role_dictionary = {}  # map of the multinode group
     if "lava-multinode" in job_data["protocols"]:
         # create target_group uuid, just a label for the coordinator.
@@ -1256,9 +1290,8 @@ def _pipeline_protocols(
             raise SubmissionException("Unable to split multinode job submission.")
 
         # structural changes done, now create the testjob.
-        parent = (
-            None
-        )  # track the zero id job as the parent of the group in the sub_id text field
+        # track the zero id job as the parent of the group in the sub_id text field
+        parent = None
         for role, role_dict in role_dictionary.items():
             for node_data in job_dictionary[role]:
                 job = _create_pipeline_job(
@@ -1278,8 +1311,8 @@ def _pipeline_protocols(
                     node_data["protocols"]["lava-multinode"]["sub_id"],
                 )
                 job.multinode_definition = (
-                    yaml_data
-                )  # store complete submission, inc. comments
+                    yaml_data  # store complete submission, inc. comments
+                )
                 job.save()
                 job_object_list.append(job)
 
@@ -1416,11 +1449,17 @@ class TestJob(models.Model):
         if self.end_time is None or self.start_time is None:
             return None
         # Only return seconds and not milliseconds
-        return datetime.timedelta(seconds=(self.end_time - self.start_time).seconds)
+        seconds = (self.end_time - self.start_time).total_seconds()
+        return datetime.timedelta(seconds=int(seconds))
 
-    STATE_SUBMITTED, STATE_SCHEDULING, STATE_SCHEDULED, STATE_RUNNING, STATE_CANCELING, STATE_FINISHED = range(
-        6
-    )
+    (
+        STATE_SUBMITTED,
+        STATE_SCHEDULING,
+        STATE_SCHEDULED,
+        STATE_RUNNING,
+        STATE_CANCELING,
+        STATE_FINISHED,
+    ) = range(6)
     STATE_CHOICES = (
         (STATE_SUBMITTED, "Submitted"),
         (STATE_SCHEDULING, "Scheduling"),
@@ -1559,7 +1598,16 @@ class TestJob(models.Model):
             self.health = TestJob.HEALTH_CANCELED
         self.state = TestJob.STATE_FINISHED
 
-        self.end_time = timezone.now()
+        # If the job is really quick to finish (a failure in validate), and
+        # lava-master is slow to response then lava-logs will notice the end of
+        # a job (and call go_state_finished) before lava-master handles the
+        # START_OK message (and call go_state_running).
+        # In this case, self.start_time would be None.
+        now = timezone.now()
+        if self.start_time is None:
+            self.start_time = now
+        self.end_time = now
+
         # TODO: check that self.actual_device is locked by the
         # select_for_update on the TestJob
         # Skip non-scheduled jobs and dynamic_connections
@@ -1657,7 +1705,7 @@ class TestJob(models.Model):
         return reverse("lava.results.testjob", args=[self.id])
 
     @property
-    def essential_role(self):  # pylint: disable=too-many-return-statements
+    def essential_role(self):
         if not self.is_multinode:
             return False
         data = yaml_safe_load(self.definition)
@@ -1674,7 +1722,7 @@ class TestJob(models.Model):
         return data["protocols"]["lava-multinode"]["essential"]
 
     @property
-    def device_role(self):  # pylint: disable=too-many-return-statements
+    def device_role(self):
         if not self.is_multinode:
             return "Error"
         try:
@@ -1869,7 +1917,7 @@ class TestJob(models.Model):
 
         # Logs.
         with contextlib.suppress(OSError):
-            data["log"] = read_logs(self.output_dir)
+            data["log"] = logs_instance.read(self)
 
         # Results.
         if results:
@@ -1878,7 +1926,7 @@ class TestJob(models.Model):
                 yaml_list = []
                 for test_case in test_suite.testcase_set.all():
                     yaml_list.append(export_testcase(test_case))
-                data["results"][test_suite.name] = yaml.dump(yaml_list)
+                data["results"][test_suite.name] = yaml_dump(yaml_list)
 
         return data
 
@@ -1985,9 +2033,8 @@ class TestJob(models.Model):
         results = {}
         attributes = [x.strip() for x in attributes.split(",")]
 
-        testdata = self.testdata_set.first()
-        if testdata:
-            for attr in testdata.attributes.all():
+        if hasattr(self, "testdata"):
+            for attr in self.testdata.attributes.all():
                 if attr.name in attributes:
                     results[attr.name] = {}
                     results[attr.name]["fail"] = self.health != self.HEALTH_COMPLETE
@@ -2006,19 +2053,36 @@ class TestJob(models.Model):
         if not xaxis_attribute:
             return None
         with contextlib.suppress(Exception):
-            testdata = self.testdata_set.first()
-            if not testdata:
+            if not hasattr(self, "testdata"):
                 return None
-            data = testdata.attributes.filter(name=xaxis_attribute)
+            data = self.testdata.attributes.filter(name=xaxis_attribute)
             return data.values_list("value", flat=True)[0]
 
     def get_metadata_dict(self):
         retval = []
-        for datum in self.testdata_set.all():
-            for attribute in datum.attributes.all():
+        if hasattr(self, "testdata"):
+            for attribute in self.testdata.attributes.all():
                 retval.append({attribute.name: attribute.value})
         return retval
 
+    @transaction.atomic
+    def cancel(self, user):
+        if not self.can_cancel(user):
+            if self.state in [TestJob.STATE_CANCELING, TestJob.STATE_FINISHED]:
+                # Don't do anything for jobs that ended already
+                return
+            raise PermissionDenied("Insufficient permissions")
+        if self.is_multinode:
+            multinode_jobs = TestJob.objects.select_for_update().filter(
+                target_group=self.target_group
+            )
+            for multinode_job in multinode_jobs:
+                multinode_job.go_state_canceling()
+                multinode_job.save()
+        else:
+            self.go_state_canceling()
+            self.save()
+
 
 class Notification(models.Model):
 
@@ -2350,3 +2414,15 @@ class GroupDevicePermission(GroupObjectPermission):
 
     def __str__(self):
         return "Permission '%s' for device %s" % (self.permission.codename, self.device)
+
+
+class GroupWorkerPermission(GroupObjectPermission):
+    class Meta:
+        unique_together = ("group", "permission", "worker")
+
+    worker = models.ForeignKey(
+        Worker, null=False, on_delete=models.CASCADE, related_name="permissions"
+    )
+
+    def __str__(self):
+        return "Permission '%s' for worker %s" % (self.permission.codename, self.worker)
