@@ -18,13 +18,13 @@
 # along with LAVA.  If not, see <http://www.gnu.org/licenses/>.
 
 import datetime
-import yaml
 
 from django.contrib.auth.models import User
 from django.db import transaction
+from django.db.models import Case, When, IntegerField, Sum
 from django.utils import timezone
 
-from lava_common.compat import yaml_safe_load
+from lava_common.compat import yaml_safe_load, yaml_safe_dump
 from lava_scheduler_app.dbutils import match_vlan_interface
 from lava_scheduler_app.models import (
     DeviceType,
@@ -35,6 +35,25 @@ from lava_scheduler_app.models import (
 )
 
 
+def worker_summary():
+    query = Worker.objects.all()
+    query = query.values("hostname", "job_limit")
+    query = query.annotate(
+        busy=Sum(
+            Case(
+                When(
+                    device__state__in=[Device.STATE_RESERVED, Device.STATE_RUNNING],
+                    then=1,
+                ),
+                default=0,
+                output_field=IntegerField(),
+            )
+        )
+    )
+    ret = {w["hostname"]: {"max": w["job_limit"], "busy": w["busy"]} for w in query}
+    return ret
+
+
 def schedule(logger, available_dt=None):
     (available_devices, jobs) = schedule_health_checks(logger, available_dt)
     jobs.extend(schedule_jobs(logger, available_devices))
@@ -46,11 +65,11 @@ def schedule_health_checks(logger, available_dt=None):
     available_devices = {}
     jobs = []
     hc_disabled = []
+    query = DeviceType.objects.filter(display=True)
     if available_dt:
         query = DeviceType.objects.filter(name__in=available_dt, display=True)
-    else:
-        query = DeviceType.objects.filter(display=True)
-    for dt in query.order_by("name"):
+    query = query.order_by("name").only("disable_health_check", "name")
+    for dt in query:
         if dt.disable_health_check:
             hc_disabled.append(dt.name)
             # Add all devices of that type to the list of available devices
@@ -88,10 +107,27 @@ def schedule_health_checks_for_device_type(logger, dt):
     )
     devices = devices.order_by("hostname")
 
+    workers_limit = worker_summary()
+
     print_header = True
     available_devices = []
     jobs = []
     for device in devices:
+        if (
+            workers_limit[device.worker_host.hostname]["max"] > 0
+            and workers_limit[device.worker_host.hostname]["busy"]
+            >= workers_limit[device.worker_host.hostname]["max"]
+        ):
+            logger.debug(
+                "SKIP healthcheck for %s due to %s having %d jobs (greater than %d)"
+                % (
+                    device.hostname,
+                    device.worker_host,
+                    workers_limit[device.worker_host.hostname]["busy"],
+                    workers_limit[device.worker_host.hostname]["max"],
+                )
+            )
+            continue
         # Do we have an health check
         health_check = device.get_health_check()
         if health_check is None:
@@ -150,6 +186,7 @@ def schedule_health_checks_for_device_type(logger, dt):
         logger.debug("  |--> scheduling health check")
         try:
             jobs.append(schedule_health_check(device, health_check))
+            workers_limit[device.worker_host.hostname]["busy"] += 1
         except Exception as exc:
             # If the health check cannot be schedule, set health to BAD to exclude the device
             logger.error("  |--> Unable to schedule health check")
@@ -212,6 +249,8 @@ def schedule_jobs_for_device_type(logger, dt, available_devices):
     # never be used.
     devices = devices.order_by("?")
 
+    workers_limit = worker_summary()
+
     jobs = []
     for device in devices:
         # Check that the device had been marked available by
@@ -219,27 +258,21 @@ def schedule_jobs_for_device_type(logger, dt, available_devices):
         # IDLE between the two functions.
         if device.hostname not in available_devices:
             continue
-        new_job = schedule_jobs_for_device(logger, device)
-        if new_job is not None:
-            jobs.append(new_job)
-    return jobs
 
-
-def schedule_jobs_for_device(logger, device):
-    jobs = TestJob.objects.filter(
-        state__in=[TestJob.STATE_SUBMITTED, TestJob.STATE_SCHEDULING]
-    )
-    jobs = jobs.filter(actual_device__isnull=True)
-    jobs = jobs.filter(requested_device_type__pk=device.device_type.pk)
-    jobs = jobs.order_by("-state", "-priority", "submit_time", "target_group", "id")
-
-    for job in jobs:
-        if not device.can_submit(job.submitter):
-            continue
-
-        device_tags = set(device.tags.all())
-        job_tags = set(job.tags.all())
-        if not job_tags.issubset(device_tags):
+        if (
+            workers_limit[device.worker_host.hostname]["max"] > 0
+            and workers_limit[device.worker_host.hostname]["busy"]
+            >= workers_limit[device.worker_host.hostname]["max"]
+        ):
+            logger.debug(
+                "SKIP %s due to %s having %d jobs (greater than %d)"
+                % (
+                    device.hostname,
+                    device.worker_host,
+                    workers_limit[device.worker_host.hostname]["busy"],
+                    workers_limit[device.worker_host.hostname]["max"],
+                )
+            )
             continue
 
         if not device.is_valid():
@@ -257,6 +290,31 @@ def schedule_jobs_for_device(logger, device):
             )
             continue
 
+        new_job = schedule_jobs_for_device(logger, device)
+        if new_job is not None:
+            jobs.append(new_job)
+            workers_limit[device.worker_host.hostname]["busy"] += 1
+    return jobs
+
+
+def schedule_jobs_for_device(logger, device):
+    jobs = TestJob.objects.filter(
+        state__in=[TestJob.STATE_SUBMITTED, TestJob.STATE_SCHEDULING]
+    )
+    jobs = jobs.filter(actual_device__isnull=True)
+    jobs = jobs.filter(requested_device_type__pk=device.device_type.pk)
+    jobs = jobs.select_related("submitter")
+    jobs = jobs.order_by("-state", "-priority", "submit_time", "target_group", "id")
+
+    device_tags = set(device.tags.all())
+    for job in jobs:
+        if not device.can_submit(job.submitter):
+            continue
+
+        job_tags = set(job.tags.all())
+        if not job_tags.issubset(device_tags):
+            continue
+
         job_dict = yaml_safe_load(job.definition)
         if "protocols" in job_dict and "lava-vland" in job_dict["protocols"]:
             if not match_vlan_interface(device, job_dict):
@@ -315,7 +373,7 @@ def transition_multinode_jobs(logger):
             # apply the complete list to all jobs in this group
             definition = yaml_safe_load(sub_job.definition)
             definition["protocols"]["lava-multinode"]["roles"] = devices
-            sub_job.definition = yaml.safe_dump(definition)
+            sub_job.definition = yaml_safe_dump(definition)
             # transition the job and device
             sub_job.go_state_scheduled()
             sub_job.save()
