@@ -197,6 +197,7 @@ def _interactive_def_schema():
                     r"^[a-zA-Z0-9-_]+$", msg=INVALID_CHARACTER_ERROR_MSG
                 ),
                 Required("prompts"): list,
+                Optional("echo"): "discard",
                 Required("script"): _interactive_script_schema(),
             }
         ]
@@ -210,7 +211,12 @@ def _interactive_script_schema():
                 Optional("name"): Match(
                     r"^[a-zA-Z0-9-_]+$", msg=INVALID_CHARACTER_ERROR_MSG
                 ),
-                Required("command"): Any(None, str),
+                Optional("command"): Any(None, str),
+                Optional("delay"): int,
+                Optional("lava-send"): str,
+                Optional("lava-sync"): str,
+                Optional("lava-wait"): str,
+                Optional("lava-wait-all"): str,
                 Optional("failures"): [
                     {
                         Required("message"): str,
@@ -232,7 +238,8 @@ def _job_command_schema():
             Required("name"): str,
             Optional("timeout"): _timeout_schema(),
             Optional("namespace"): str,
-        }
+        },
+        extra=True,
     )
 
 
@@ -349,7 +356,7 @@ def _validate_multinode(data_object):
                 )
             if expect_role not in roles:
                 raise SubmissionException(
-                    "'expect_role' '%s' does not exist" % host_role
+                    "'expect_role' '%s' does not exist" % expect_role
                 )
         elif expect_role is not None:
             raise SubmissionException("'expect_role' without 'host_role'")
@@ -393,27 +400,26 @@ def visibility_schema():
     return Schema(Any("public", "personal", {"group": [str]}))
 
 
-def _job_schema():
-    metadata_types = Any(str, int)
-    return Schema(
-        {
-            "device_type": All(
-                str, Length(min=1)
-            ),  # not Required as some protocols encode it elsewhere
-            Required("job_name"): All(str, Length(min=1, max=200)),
-            Optional("priority"): Any("high", "medium", "low", int),
-            Optional("protocols"): _job_protocols_schema(),
-            Optional("context"): _context_schema(),
-            Optional("metadata"): All({metadata_types: metadata_types}),
-            Optional("secrets"): dict,
-            Optional("tags"): [str],
-            Required("visibility"): visibility_schema(),
-            Required("timeouts"): _job_timeout_schema(),
-            Required("actions"): _job_actions_schema(),
-            Optional("notify"): _job_notify_schema(),
-            Optional("reboot_to_fastboot"): bool,
-        }
-    )
+_job_schema = Schema(
+    {
+        "device_type": All(
+            str, Length(min=1)
+        ),  # not Required as some protocols encode it elsewhere
+        Required("job_name"): All(str, Length(min=1, max=200)),
+        Optional("priority"): Any("high", "medium", "low", int),
+        Optional("protocols"): _job_protocols_schema(),
+        Optional("context"): _context_schema(),
+        Optional("metadata"): All({Any(str, int): Any(str, int)}),
+        Optional("secrets"): dict,
+        Optional("environment"): dict,
+        Optional("tags"): [str],
+        Required("visibility"): visibility_schema(),
+        Required("timeouts"): _job_timeout_schema(),
+        Required("actions"): _job_actions_schema(),
+        Optional("notify"): _job_notify_schema(),
+        Optional("reboot_to_fastboot"): bool,
+    }
+)
 
 
 def _device_deploy_schema():
@@ -461,40 +467,36 @@ def _device_commands_schema():
     )
 
 
-def _device_schema():
-    """
-    Less strict than the job_schema as this is primarily admin / template controlled.
-    """
-    return Schema(
-        {
-            "character_delays": dict,
-            "commands": _device_commands_schema(),
-            "constants": dict,
-            "adb_serial_number": str,
-            "fastboot_serial_number": str,
-            "fastboot_options": [str],
-            "fastboot_via_uboot": bool,
-            "device_info": [dict],
-            "static_info": [dict],
-            "storage_info": [dict],
-            "environment": dict,
-            "flash_cmds_order": list,
-            "parameters": dict,
-            "board_id": str,
-            "usb_vendor_id": All(
-                str, Length(min=4, max=4)
-            ),  # monitor type like arduino
-            "usb_product_id": All(
-                str, Length(min=4, max=4)
-            ),  # monitor type like arduino
-            "usb_sleep": int,
-            "usb_filesystem_label": str,
-            "usb_serial_driver": str,
-            "actions": _device_actions_schema(),
-            "timeouts": _device_timeouts_schema(),
-            "available_architectures": list,
-        }
-    )
+# Less strict than the job_schema as this is primarily admin / template
+# controlled.
+_device_schema = Schema(
+    {
+        "character_delays": dict,
+        "commands": _device_commands_schema(),
+        "constants": dict,
+        "adb_serial_number": str,
+        "fastboot_serial_number": str,
+        "fastboot_options": [str],
+        "fastboot_via_uboot": bool,
+        "device_info": [dict],
+        "static_info": [dict],
+        "storage_info": [dict],
+        "environment": dict,
+        "flash_cmds_order": list,
+        "parameters": dict,
+        "board_id": str,
+        "usb_vendor_id": All(str, Length(min=4, max=4)),  # monitor type like arduino
+        "usb_product_id": All(str, Length(min=4, max=4)),  # monitor type like arduino
+        "usb_sleep": int,
+        "usb_filesystem_label": str,
+        "usb_serial_driver": str,
+        "actions": _device_actions_schema(),
+        "timeouts": _device_timeouts_schema(),
+        "available_architectures": list,
+        "uuu_usb_otg_path": str,
+        "uuu_corrupt_boot_media_command": [str],
+    }
+)
 
 
 def _validate_secrets(data_object):
@@ -526,8 +528,7 @@ def validate_submission(data_object):
     :return: True if valid, else raises SubmissionException
     """
     try:
-        schema = _job_schema()
-        schema(data_object)
+        _job_schema(data_object)
     except MultipleInvalid as exc:
         raise SubmissionException(exc)
 
@@ -562,8 +563,7 @@ def validate_device(data_object):
     :return: True if valid, else raises SubmissionException
     """
     try:
-        schema = _device_schema()
-        schema(data_object)
+        _device_schema(data_object)
     except MultipleInvalid as exc:
         raise SubmissionException(exc)
 
