@@ -18,7 +18,9 @@
 # You should have received a copy of the GNU Affero General Public License
 # along with LAVA.  If not, see <http://www.gnu.org/licenses/>.
 
+import contextlib
 import os
+import pathlib
 import xmlrpc.client
 import yaml
 
@@ -27,8 +29,8 @@ from django.conf import settings
 from django.contrib.auth.models import Group
 from django.core.exceptions import PermissionDenied
 
-from lava_common.compat import yaml_safe_load
-from lava_common.utils import debian_package_version
+from lava_common.compat import yaml_safe_dump, yaml_safe_load
+from lava_common.version import __version__
 from lava_scheduler_app.api import check_perm
 from lava_scheduler_app.views import get_restricted_job
 from lava_scheduler_app.models import (
@@ -86,7 +88,7 @@ class LavaSystemAPI(SystemAPI):
         ------------
         lava-server version string
         """
-        return debian_package_version(pkg="lava-server")
+        return __version__
 
     # Update the integer return value when adding arguments to
     # existing functions anywhere in the XML-RPC API or
@@ -513,14 +515,14 @@ class LavaSystemAPI(SystemAPI):
 
         if switch:
             if switch in network_map["switches"]:
-                return yaml.dump(network_map["switches"][switch])
+                return yaml_safe_dump(network_map["switches"][switch])
             else:
                 return xmlrpc.client.Fault(
                     404,
                     "No switch '%s' was found in the network map of supported devices."
                     % switch,
                 )
-        return yaml.dump(network_map)
+        return yaml_safe_dump(network_map)
 
     @check_perm("lava_scheduler_app.change_devicetype")
     def assign_perm_device_type(self, perm, device_type, group):
@@ -721,6 +723,34 @@ class LavaSystemAPI(SystemAPI):
 
         GroupDevicePermission.objects.remove_perm(perm, group, device)
 
+    def get_master_certificate(self):
+        """
+        Name
+        ----
+        `scheduler.system.get_master_certificate` ()
+
+        Description
+        -----------
+        Return the master public certificate found in location specified
+        in django settings.
+
+        Arguments
+        ---------
+        None
+
+        Return value
+        ------------
+        This function returns the public certificate found on master
+        """
+        self._authenticate()
+
+        path = pathlib.Path(settings.MASTER_CERT_PUB)
+        with contextlib.suppress(OSError):
+            data = path.read_text(encoding="utf-8")
+            return xmlrpc.client.Binary(data.encode("utf-8"))
+
+        raise xmlrpc.client.Fault(404, "Master does not have a certificate")
+
 
 class LavaMapper(Mapper):
     def register_introspection_methods(self):
