@@ -23,11 +23,12 @@
 # Pre install some usefull tools when stating a python script with:
 # PYTHONSTARTUP=./share/startup.py python
 #
-import django
-from dashboard_app.models import *
+
+import django  # pylint: disable=unused-import
 from lava_scheduler_app.models import *
 from linaro_django_xmlrpc.models import *
 from django.db import transaction
+
 print("=============================")
 print("Startup script for LAVA")
 
